﻿#include "LoginWindow.h"
#include "ui_loginwindow.h"

#include <QGraphicsDropShadowEffect>
#include <QPainter>
#include <QLineEdit>
#include <QDebug>
#include <QRegExpValidator>
#include <QMessageBox>
#include <QMouseEvent>
#include <QCheckBox>

#include "QAlerDialog.h"
#include "XYSettings.h"


#pragma execution_character_set("utf-8")

LoginWindow::LoginWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::LoginWindow)
{
    ui->setupUi(this);
    QGraphicsDropShadowEffect *shadowEffect = new QGraphicsDropShadowEffect(this);
    shadowEffect->setOffset(0, 0);
    shadowEffect->setColor(Qt::gray);
    shadowEffect->setBlurRadius(10);
    ui->frame->setGraphicsEffect(shadowEffect);
    this->setAttribute(Qt::WA_TranslucentBackground);

    // load settings
    loadSetting(true);

    ui->btnJoin->setEnabled(false);

    QFile file(":/stylesheet/style_buttom.qss");
    if (file.open(QFile::ReadOnly)) {
        this->setStyleSheet(tr(file.readAll()));
        file.close();
    }

    ui->txtRandom->installEventFilter(this);

//    QRegExp regx("[1-9][0-9]+$");
//    QValidator *validator = new QRegExpValidator(regx, ui->edtNumber);
//    ui->edtNumber->setValidator(validator);

    ui->edtNumber->setText("");
    ui->tv_name->setText("");
    ui->tv_uid->setText("");

    qRegisterMetaType<SDKVideoFrame>("SDKVideoFrame");
    qRegisterMetaType<SDKCallInfo>("SDKCallInfo");
    qRegisterMetaType<SDKLoginInfo>("SDKLoginInfo");
    qRegisterMetaType<SDKLoginState>("SDKLoginState");

    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigLoginResult, this, &LoginWindow::onLoginResult);
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigKickOut, this, &LoginWindow::onKickOut);
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigCallInvited, this, &LoginWindow::onCallInvited);
    connect(ui->tbtnSetting, &QToolButton::clicked, this, &LoginWindow::onSettingClicked);
    connect(ui->pb_logout, &QPushButton::clicked, this, &LoginWindow::onLogoutClicked);

    ui->stackedWidget->setCurrentIndex(0);
    connect(ui->bt_login, &QPushButton::clicked, this, &LoginWindow::onLoginClicked);
    connect(ui->cb_xy_acount_login, &QCheckBox::clicked, this, &LoginWindow::onLoginTypeClicked);
}

LoginWindow::~LoginWindow()
{
    delete ui;
}

void LoginWindow::onLoginResult(SDKLoginState state, const QString& code, const SDKLoginInfo& info)
{
    if (state == SDKLoginState_LoggedIn) {
        if (code == SDKError_NoError) {
            ui->stackedWidget->setCurrentIndex(1);
            _displayName = info.displayName.str();
        } else {
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, QString("登录失败, error: %1").arg(
                                  NemoSdkAdaptor::getInstance()->error_info(code)));
            delete dialog;
        }
    }
    if (state == SDKLoginState_LoggedOut) {
        if (code == SDKError_NoError) {
            ui->stackedWidget->setCurrentIndex(0);
            loadSetting();
        } else {
            qDebug() << "-----> " << NemoSdkAdaptor::getInstance()->error_info(code);
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, QString("操作失败, error: %1").arg(
                                  NemoSdkAdaptor::getInstance()->error_info(code)));
            delete dialog;
        }
    }
}

void LoginWindow::onCallInvited(const SDKCallInfo& info)
{
    this->hide();
    meetingWindow->show();
    meetingWindow->onMeetingInvited(_displayName, info, !ui->cbCamera->isChecked(), !ui->cbMic->isChecked());
}

void LoginWindow::onConnectionChanged(bool state)
{
    qDebug() << "onConnectionChanged : " << state;
    ui->btnJoin->setEnabled(state);
}

void LoginWindow::onKickOut(const QString& code)
{
    qDebug() << "onKickOut : " << code;
    NemoSdkAdaptor::getInstance()->logout();
}


void LoginWindow::mousePressEvent(QMouseEvent *e)
{
    last = e->globalPos();
}

void LoginWindow::mouseMoveEvent(QMouseEvent *e)
{
    int dx = e->globalX() - last.x();
    int dy = e->globalY() - last.y();
    last = e->globalPos();
    move(x()+dx, y()+dy);
}

void LoginWindow::mouseReleaseEvent(QMouseEvent *e)
{
    int dx = e->globalX() - last.x();
    int dy = e->globalY() - last.y();
    move(x()+dx, y()+dy);
}

void LoginWindow::onCallFinished(QString reason)
{
    meetingWindow->close();
    meetingWindow->deleteLater();
    this->show();
}

void LoginWindow::on_btnJoin_clicked()
{
    if(NemoSdkAdaptor::getInstance()->isConnected()) {
        this->hide();
        meetingWindow = new MeetingWindow();
        meetingWindow->show();
        meetingWindow->setUIShowLocalWhenPageMode(true);
        meetingWindow->makeCall(ui->edtNumber->text(), _displayName, "", !ui->cbCamera->isChecked(),
                                !ui->cbMic->isChecked(), ui->tv_meeting_id->text());
        connect(meetingWindow, &MeetingWindow::sigMeetingFinished, this, &LoginWindow::onCallFinished);
    } else {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("Network unavailable"));
        delete dialog;
    }
}

void LoginWindow::on_tbtnClose_clicked()
{
    if (!NemoSdkAdaptor::getInstance()->isSdkClosed()){
        NemoSdkAdaptor::getInstance()->shutdown();
    }
    this->close();
}

void LoginWindow::on_tbtnMinise_clicked()
{
    this->showMinimized();
}

void LoginWindow::on_edtNumber_textChanged(const QString &arg1)
{
    ui->btnJoin->setEnabled(arg1.length() > 0);
}

void LoginWindow::onLoginClicked()
{
    qDebug() << "onLoginClicked: " << extId;
    QString displayName = ui->tv_name->text();
    QString uuid = ui->tv_uid->text();
    if (displayName.isEmpty() || uuid.isEmpty()) {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("入会名称/账号不能为空"));
        delete dialog;
        return;
    }
    if (ui->cb_xy_acount_login->isChecked()) {
        NemoSdkAdaptor::getInstance()->login(displayName.toStdString(), uuid.toStdString());
    } else {
        QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
        QSettings configs(settingConfig, QSettings::IniFormat);
        configs.beginGroup("serverConfig");
        QString extId = configs.value("extId").toString();
        NemoSdkAdaptor::getInstance()->loginWithExtId(extId.toStdString(), uuid.toStdString(), displayName.toStdString());
    }

    // 免密快速登录
    //NemoSdkAdaptor::getInstance()->quickLogin(enterpriseId.toStdString().c_str(), displayName.toStdString().c_str());
}

void LoginWindow::onSettingClicked()
{
    XYSettings* settings = new XYSettings();
    settings->setAttribute(Qt::WA_DeleteOnClose);
    settings->show();
    connect(settings, &XYSettings::sigServerChanged, this, &LoginWindow::onServerChanged);
}

/**
 * server changed need re-login
 *
 * @brief LoginWindow::onServerChanged
 * @param host
 */
void LoginWindow::onServerChanged()
{
    qDebug() << __func__;
    if (ui->stackedWidget->currentIndex() == 1) {
        // 登录状态下需要登出
        NemoSdkAdaptor::getInstance()->logout();
        ui->stackedWidget->setCurrentIndex(0); // test remove
        loadSetting();
    } else {
        loadSetting();
    }
}

void LoginWindow::onLogoutClicked()
{
    NemoSdkAdaptor::getInstance()->logout();
}

void LoginWindow::onLoginTypeClicked() {
    if (ui->cb_xy_acount_login->isChecked()) {
        ui->tv_name->setPlaceholderText(tr("手机号/邮箱"));
        ui->tv_uid->setPlaceholderText(tr("密码"));
    } else {
        ui->tv_name->setPlaceholderText(tr("输入显示名称"));
        ui->tv_uid->setPlaceholderText(tr("输入用户ID"));
    }
}

void LoginWindow::loadSetting(bool firstInit)
{
    QString settingConfig = QCoreApplication::applicationDirPath() + "/config.ini";
    qDebug() << "settingConfig: " << settingConfig;
    QSettings configs(settingConfig, QSettings::IniFormat);
    configs.beginGroup("serverConfig");
    QString host = configs.value("host").toString();
    QString sysType = configs.value("sysType").toString();
    QString clientId = configs.value("clientId").toString();
    QString clientSecret = configs.value("clientSecret").toString();
    QString extId = configs.value("extId").toString();
    if (host.isEmpty()) {
        // set default config
        configs.setValue("host", "sdkapi.xylink.com");
        host = "sdkapi.xylink.com";
    }
    if (sysType.isEmpty()) {
        // set default sysType
        configs.setValue("sysType", "c++sdk");
    }
    if (firstInit) {
        if (clientId.isEmpty()) {
            // set default clientId
            configs.setValue("clientId", "l9Asj75NryPJaK3sshFFksuo");
            clientId = "l9Asj75NryPJaK3sshFFksuo";
        }
    }

    if (clientSecret.isEmpty()) {
        // set default clientSecret
        configs.setValue("clientSecret", "tUvFBtsIiST7oFn31Tap781qC5kFcfXH");
        clientSecret = "tUvFBtsIiST7oFn31Tap781qC5kFcfXH";
    }
    if (extId.isEmpty()) {
        // set default extId
        configs.setValue("extId", "0142901e3d83e0a1e225ef92b8663fcaebda7242");
        extId = "0142901e3d83e0a1e225ef92b8663fcaebda7242";
    }
    qDebug() << "===>" << host << clientId << clientSecret;
    NemoSdkAdaptor::getInstance()->init(host, extId, clientId, clientSecret);
}

bool LoginWindow::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == ui->txtRandom && event->type() == QEvent::MouseButtonPress) {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, ("暂未开发"));
        delete dialog;
        return true;
    }
    return QWidget::eventFilter(obj, event);
}

void LoginWindow::on_cbCamera_clicked()
{

}
