﻿#include "XYSettings.h"
#include "ui_XYSettings.h"

#include <QRegExp>
#include <QRegExpValidator>
#include <QSettings>
#include <QButtonGroup>
#include <QDebug>
#include <QTimer>
#include <QVBoxLayout>
#include <QDir>

#include "QAlerDialog.h"
#include "meeting/MenuButton.h"

#pragma execution_character_set("utf-8")

struct MediaDevInfo {
    std::string devId;
    std::string devName;
    bool isSelected;
};
Q_DECLARE_METATYPE(MediaDevInfo);

XYSettings::XYSettings(QDialog *parent, const char *newParameter, bool incall) :
    QDialog(parent),
    ui(new Ui::XYSettings),
    isPressed(false),
    bInCall(incall)
{
    ui->setupUi(this);
    this->setWindowFlags(Qt::FramelessWindowHint);
    this->setModal(true);
    this->setAttribute(Qt::WA_DeleteOnClose);

    QFile file(":/stylesheet/style_setting.qss");
    if (file.open(QFile::ReadOnly)) {
        this->setStyleSheet(tr(file.readAll()));
        file.close();
    }

    ui->pb_mic_check->setVisible(!bInCall);
    ui->pb_speaker_check->setVisible(!bInCall);

    // load settings
    QString settingConfig = QCoreApplication::applicationDirPath();
    settingConfig += "/config.ini";
    configs = new QSettings(settingConfig, QSettings::IniFormat, this);
    configs->beginGroup("serverConfig");
    updateUi();
    lastServerConfig = ui->et_host->text();
    lastClientIdConfig = ui->et_clietId->text();

    toolBar = new QButtonGroup(this);
    toolBar->addButton(ui->tb_setting_common, COMMON);
    toolBar->addButton(ui->tb_setting_audio, AUDIO);
    toolBar->addButton(ui->tb_setting_video, VIDEO);
    toolBar->addButton(ui->tb_setting_beauty, BEAUTY);
    toolBar->addButton(ui->tb_setting_feedback, FEEDBACK);
    toolBar->setExclusive(true);

    ui->pagerContainer->setCurrentIndex(COMMON);
    ui->tb_setting_common->setChecked(Qt::Checked);

    QButtonGroup* audioCheckGroup = new QButtonGroup(this);
    audioCheckGroup->addButton(ui->pb_speaker_check, SPEAKER);
    audioCheckGroup->addButton(ui->pb_mic_check, MIC);
    audioCheckGroup->setExclusive(true);

    // setting video resolution & frame
    QStringList resolution = {"180P", "360P", "720P", "1080P"};
    ui->video_resolution->addItems(resolution);
    ui->video_frame->addItems({"15fps", "30fps"});
    ui->video_resolution->setCurrentIndex(R_720P);

    // check timer
    speakerTimer = new QTimer(this);
    micTimer = new QTimer(this);
    openGL = new OpenGLDisplay(this);
    ui->gl_container->addWidget(openGL);

    qRegisterMetaType<SDKVideoFrameWrapper>("SDKVideoFrameWrapper");
    qRegisterMetaType<SDKMediaDevType>("SDKMediaDevType");
    qRegisterMetaType<SDKHttpResponse>("SDKHttpResponse");
    qRegisterMetaType<SDKRestApiResult>("SDKRestApiResult");

    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigDeviceChanged, this, &XYSettings::onDeviceChanged);
    connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigLoguploadResponse, this, &XYSettings::onLoguploadResponse);

    connect(ui->bt_close, &QToolButton::clicked, this, &QToolButton::close);
    connect(ui->bt_save, &QPushButton::clicked, this, &XYSettings::onSaveClicked);
    connect(ui->bt_reset, &QPushButton::clicked, this, &XYSettings::onResetClicked);
    //connect(toolBar, QOverload<int>::of(&QButtonGroup::buttonClicked), this, &XYSettings::onSettingButtonsChecked);
    connect(toolBar, SIGNAL(buttonClicked(int)), this, SLOT(onSettingButtonsChecked(int)));
    connect(ui->slider_volume_out, &QSlider::valueChanged, this, &XYSettings::onSliderVolumeOutChanged);
    //connect(audioCheckGroup, QOverload<int>::of(&QButtonGroup::buttonClicked), this, &XYSettings::onAudioCheckGroupClicked);
    connect(audioCheckGroup, SIGNAL(buttonClicked(int)), this, SLOT(onAudioCheckGroupClicked(int)));
    //connect(ui->cb_speaker, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &XYSettings::onSpeakerSelected);
    connect(ui->cb_speaker, SIGNAL(currentIndexChanged(int)), this, SLOT(onSpeakerSelected()));
    //connect(ui->cb_mic, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &XYSettings::onMicSelected);
    connect(ui->cb_mic, SIGNAL(currentIndexChanged(int)), this, SLOT(onMicSelected()));
    //connect(ui->cb_camera, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &XYSettings::onCameraSelected);
    connect(ui->cb_camera, SIGNAL(currentIndexChanged(int)), this, SLOT(onCameraSelected()));
    //connect(ui->video_resolution, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &XYSettings::onVideoResolutionChanged);
    //connect(ui->video_frame, QOverload<int>::of(&QComboBox::currentIndexChanged), this, &XYSettings::onVideoFrameChanged);
    connect(ui->pb_video_setting_confirm, &QPushButton::clicked, this, &XYSettings::onVideoSettingConfirmClicked);

    // 虚拟背景&美颜&滤镜
    beautyOpenGL = new OpenGLDisplay(ui->video_container);
    _needCloseCamera = NemoSdkAdaptor::getInstance()->isCameraClosed();
    fl = new FrameLayout(this, 0, 0, 0);
    fl->addWidget(beautyOpenGL);
    connect(ui->tabWidget, &QTabWidget::tabBarClicked, this, &XYSettings::onBeautyTabClicked);
    initVirtualBg();
    initBeauty();
    initFilter();
    ui->video_container->setLayout(fl);

    ui->tabWidget->setTabText(Tab_VirtualBg, "虚拟背景");
    ui->tabWidget->setTabText(Tab_Beauty, "美颜");
    ui->tabWidget->setTabText(Tab_Filter, "滤镜");
    ui->tabWidget->setCurrentIndex(Tab_VirtualBg);


    ui->tv_upload_tips->setVisible(false);
    connect(ui->bt_log_upload, &QPushButton::clicked, this, [=]() {
        if (!NemoSdkAdaptor::getInstance()->isLogedIn()) {
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, ("请先登录"));
            delete dialog;
            return;
        }
        QString logComment = ui->et_log_comment->text();
        if (logComment.isEmpty()) {
            QAlerDialog *dialog = new QAlerDialog();
            dialog->showAlert(QAlerDialog::TYPE_TIPS, ("请输入意见和建议"));
            delete dialog;
        } else {
            ui->tv_upload_tips->setVisible(true);
            ui->bt_log_upload->setEnabled(false);
            NemoSdkAdaptor::getInstance()->logUpload(logComment.toStdString());
        }
    });
}

void XYSettings::onLoguploadResponse(const SDKRestApiResult &result)
{
    ui->bt_log_upload->setEnabled(true);
    ui->tv_upload_tips->setVisible(false);
    QAlerDialog *dialog = new QAlerDialog();
    dialog->showAlert(QAlerDialog::TYPE_TIPS, ("上传成功, 感谢您的反馈"));
    delete dialog;
}

XYSettings::~XYSettings()
{
    NemoSdkAdaptor::getInstance()->enablePulseHelper(false);
    if (_needCloseCamera && !NemoSdkAdaptor::getInstance()->isCameraClosed()) {
        NemoSdkAdaptor::getInstance()->stopCamera();
    }
    delete ui;
}

void XYSettings::updateUi()
{
    ui->et_host->setText(configs->value("host").toString());
    ui->et_clietId->setText(configs->value("clientId").toString());
    ui->et_client_secret->setText(configs->value("clientSecret").toString());
    ui->et_enterpriseId->setText(configs->value("extId").toString());
}

void XYSettings::onSettingButtonsChecked(int id)
{
    if (id == ui->pagerContainer->currentIndex()) {
        return;
    }
    if (id != VIDEO && id != BEAUTY) {
        disconnect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigVideoFrameReady, this, &XYSettings::onVideoFrameReady);
        if (_needCloseCamera && !NemoSdkAdaptor::getInstance()->isCameraClosed()) {
            NemoSdkAdaptor::getInstance()->stopCamera();
        }
    } else {
        connect(NemoSdkAdaptor::getInstance(), &NemoSdkAdaptor::sigVideoFrameReady, this, &XYSettings::onVideoFrameReady);
    }
    if ((id == VIDEO && ui->pagerContainer->currentIndex() != BEAUTY) ||
            (id == BEAUTY && ui->pagerContainer->currentIndex() != VIDEO))
    {
        if (NemoSdkAdaptor::getInstance()->isCameraClosed()) {
            NemoSdkAdaptor::getInstance()->startCamera();
        }
    }
    if (id != AUDIO) {
        if (startMicCheck) {
            ui->pb_mic_check->setText(tr("检测麦克风"));
            NemoSdkAdaptor::getInstance()->enableListnerTest(false);
            startMicCheck = false;
            timerCheckEvent(MIC, startMicCheck);
        }
        if (startSpeakerCheck) {
            ui->pb_speaker_check->setText(tr("检测扬声器"));
            ui->lb_speaker_tips->setVisible(false);
            NemoSdkAdaptor::getInstance()->stopPlayWavfile();
            startSpeakerCheck = false;
            timerCheckEvent(SPEAKER, startSpeakerCheck);
        }
        NemoSdkAdaptor::getInstance()->enablePulseHelper(false);
    }
    switch (id) {
    case COMMON:
        ui->pagerContainer->setCurrentIndex(COMMON);
        ui->tb_setting_common->setChecked(true);
        break;
    case AUDIO:
        ui->pagerContainer->setCurrentIndex(AUDIO);
        ui->lb_speaker_tips->setVisible(false);
        ui->tb_setting_audio->setChecked(true);
        ui->slider_volume_out->setValue(NemoSdkAdaptor::getInstance()->getSpeakerVolume());
        NemoSdkAdaptor::getInstance()->enablePulseHelper(true);
        getAudioDevices();
        break;
    case VIDEO:
        ui->pagerContainer->setCurrentIndex(VIDEO);
        ui->tb_setting_video->setChecked(true);
        getVideoDevices();
        break;
    case FEEDBACK:
        ui->pagerContainer->setCurrentIndex(FEEDBACK);
        ui->tb_setting_feedback->setChecked(true);
        break;
    case BEAUTY:
        ui->pagerContainer->setCurrentIndex(BEAUTY);
        ui->tb_setting_beauty->setChecked(true);
        break;
    }
}

void XYSettings::onAudioCheckGroupClicked(int id)
{
    switch (id) {
    case SPEAKER:
        if (startMicCheck) {
            ui->pb_mic_check->setText(tr("检测麦克风"));
            NemoSdkAdaptor::getInstance()->enableListnerTest(false);
            startMicCheck = false;
        }
        startMicCheck = false;
        startSpeakerCheck = !startSpeakerCheck;
        if (startSpeakerCheck) {
            ui->pb_speaker_check->setText(tr("停止"));
#ifdef DEBUG
            QString filePath = QCoreApplication::applicationDirPath() + "/ring.wav";
#else
             QString filePath = "/opt/apps/com.xylink.sdkdemo/files/xylink/icon/ring.wav";
#endif
            NemoSdkAdaptor::getInstance()->startPlayWavFile(filePath.toLocal8Bit().constData(), 0);
            connect(speakerTimer, &QTimer::timeout, this, [=]() {
                ui->pb_volume_out->setValue(NemoSdkAdaptor::getInstance()->getSpeakerPeakMeter());
            });
        } else {
            ui->pb_speaker_check->setText(tr("检测扬声器"));
            NemoSdkAdaptor::getInstance()->stopPlayWavfile();
        }
        ui->lb_speaker_tips->setVisible(startSpeakerCheck);
        timerCheckEvent(SPEAKER, startSpeakerCheck);
        break;
    case MIC:
        if (startSpeakerCheck) {
            ui->pb_speaker_check->setText(tr("检测扬声器"));
            ui->lb_speaker_tips->setVisible(false);
            NemoSdkAdaptor::getInstance()->stopPlayWavfile();
            startSpeakerCheck = false;
        }
        startMicCheck = !startMicCheck;
        if (startMicCheck) {
            ui->pb_mic_check->setText(tr("停止"));
            NemoSdkAdaptor::getInstance()->enableListnerTest(true);
            connect(micTimer, &QTimer::timeout, this, [=]() {
                ui->pb_volume_in->setValue(NemoSdkAdaptor::getInstance()->getMicPeakMeter());
            });
        } else {
            NemoSdkAdaptor::getInstance()->enableListnerTest(false);
            ui->pb_mic_check->setText(tr("检测麦克风"));
        }
        timerCheckEvent(MIC, startMicCheck);
        break;
    }
}

void XYSettings::timerCheckEvent(XYSettings::CheckType checkType, bool isStart)
{
    switch (checkType) {
    case SPEAKER:
        if (micTimer->isActive()) {
            micTimer->stop();
        }
        if (speakerTimer->isActive()) {
            if (!isStart) {
                speakerTimer->stop();
            }
        } else {
            if (isStart) {
                speakerTimer->start(100);
            }
        }
        break;
    case MIC:
        if (speakerTimer->isActive()) {
            speakerTimer->stop();
        }
        if (micTimer->isActive()) {
            if (!isStart) {
                micTimer->stop();
            }
        } else {
            if (isStart) {
                micTimer->start(100);
            }
        }
        break;
    }
}

void XYSettings::onVideoFrameReady(const QString &dataSourceId, const SDKVideoFrameWrapper& videoFrame)
{
    if (NemoSdkAdaptor::getInstance()->isSdkClosed()) {
        return;
    }
    if(dataSourceId == "LocalPreviewID")
    {
        bool isFlip = ui->check_camera_mirror->isChecked();
        if (openGL->isVisible()) {
            openGL->initBuffer(1280 * 720 * 3 / 2);
            openGL->requestRender(videoFrame.buffer, videoFrame.bufLen, videoFrame.width, videoFrame.height,
                                  videoFrame.sarWidth, videoFrame.sarHeight, videoFrame.rotation, isFlip);
        }
        if (beautyOpenGL->isVisible()) {
            beautyOpenGL->initBuffer(1280 * 720 * 3 / 2);
            beautyOpenGL->requestRender(videoFrame.buffer, videoFrame.bufLen, videoFrame.width, videoFrame.height,
                                        videoFrame.sarWidth, videoFrame.sarHeight, videoFrame.rotation, isFlip);
        }
    }
}

void XYSettings::mouseMoveEvent (QMouseEvent * event) {
    if (isPressed) {
        QPoint movePos = event->globalPos() - m_Position;
        move(movePos);
        event->accept();
    }
}

void XYSettings::mousePressEvent (QMouseEvent * event) {
    if (event->button() == Qt::LeftButton) {
        m_Position = event->pos();
        int titleWidth = ui->topbar->height();
        if (m_Position.y() < titleWidth) {
            isPressed = true;
        }
    }
}

void XYSettings::mouseReleaseEvent(QMouseEvent *) {
    isPressed = false;
}

void XYSettings::onSaveClicked()
{
    // set config
    QString host = ui->et_host->text();
    QString clientId = ui->et_clietId->text();
    QString clientSecret = ui->et_client_secret->text();
    QString extId = ui->et_enterpriseId->text();

    if (host.isEmpty() || clientSecret.isEmpty() || extId.isEmpty()) {
        QAlerDialog *dialog = new QAlerDialog();
        dialog->showAlert(QAlerDialog::TYPE_TIPS, tr("请检测输入是否正确!!!"));
        delete dialog;
        return;
    }

    if (lastServerConfig == host && lastClientIdConfig == ui->et_clietId->text()) {
        close();
        return;
    }
    configs->setValue("host", host);
    configs->setValue("clientId", clientId);
    configs->setValue("clientSecret", clientSecret);
    configs->setValue("extId", extId);
    emit sigServerChanged();
    close();
}

void XYSettings::onResetClicked()
{
    configs->setValue("host", "sdkapi.xylink.com");
    configs->setValue("clientId", "l9Asj75NryPJaK3sshFFksuo");
    configs->setValue("clientSecret", "tUvFBtsIiST7oFn31Tap781qC5kFcfXH");
    configs->setValue("extId", "0142901e3d83e0a1e225ef92b8663fcaebda7242");
    updateUi();
}

void XYSettings::onSliderVolumeInChanged(int value)
{
    NemoSdkAdaptor::getInstance()->setMicVolume(value);
}

void XYSettings::onSliderVolumeOutChanged(int value)
{
    NemoSdkAdaptor::getInstance()->setSpeakerVolume(value);
}

void XYSettings::getAudioDevices()
{
    ui->cb_speaker->clear();
    XYArray<SDKMediaDevInfo> speakerDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Speaker);
    if (speakerDeviceList.length() > 0) {
        for (int i = 0; i < speakerDeviceList.length(); i++) {
            if (!QString(speakerDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = speakerDeviceList[i].devId.str();
                mediaInfo.devName = speakerDeviceList[i].devName.str();
                mediaInfo.isSelected = speakerDeviceList[i].isSelected;
                ui->cb_speaker->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected)
                    ui->cb_speaker->setCurrentText(mediaInfo.devName.c_str());
            }
        }
    }
    ui->cb_mic->clear();
    XYArray<SDKMediaDevInfo> micDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Microphone);
    if (micDeviceList.length() > 0) {
        for (int i = 0; i < micDeviceList.length(); i++) {
            if (!QString(micDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = micDeviceList[i].devId.str();
                mediaInfo.devName = micDeviceList[i].devName.str();
                mediaInfo.isSelected = micDeviceList[i].isSelected;
                ui->cb_mic->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected)
                    ui->cb_mic->setCurrentText(mediaInfo.devName.c_str());
            }
        }
    }
}

void XYSettings::getVideoDevices()
{
    ui->cb_camera->clear();
    XYArray<SDKMediaDevInfo> cameraDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Camera);
    if (cameraDeviceList.length() > 0) {
        for (int i = 0; i < cameraDeviceList.length(); i++) {
            if (!QString(cameraDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = cameraDeviceList[i].devId.str();
                mediaInfo.devName = cameraDeviceList[i].devName.str();
                mediaInfo.isSelected = cameraDeviceList[i].isSelected;
                ui->cb_camera->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected) {
                    ui->cb_camera->setCurrentText(mediaInfo.devName.c_str());
                }
            }
        }
    }
}

void XYSettings::onSpeakerSelected()
{
    QVariant speakerData =ui->cb_speaker->currentData(Qt::UserRole);
    if (speakerData.isValid()) {
        MediaDevInfo speakerInfo = speakerData.value<MediaDevInfo>();
        NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Speaker, speakerInfo.devId.c_str());
        qDebug() << "cbo chooseDev speaker: " << speakerInfo.devName.c_str() << ", id: " << speakerInfo.devId.c_str();
    }
}

void XYSettings::onMicSelected()
{
    QVariant micData =ui->cb_mic->currentData(Qt::UserRole);
    if (micData.isValid()) {
        MediaDevInfo micInfo = micData.value<MediaDevInfo>();
        NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Microphone, micInfo.devId.c_str());
        qDebug() << "cbo chooseDev micInfo: " << micInfo.devName.c_str() << ", id: " << micInfo.devId.c_str();
    }
}

void XYSettings::onCameraSelected()
{
    QVariant cameraData =ui->cb_camera->currentData(Qt::UserRole);
    if (cameraData.isValid()) {
        MediaDevInfo cameraInfo = cameraData.value<MediaDevInfo>();
        qDebug() << __FUNCTION__ << ", isSelected: " << cameraInfo.isSelected << ", devId: " << cameraInfo.devId.c_str() << ", devName: " << cameraInfo.devName.c_str();
        NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Camera, cameraInfo.devId.c_str());
    }
}

void XYSettings::onVideoResolutionChanged(int index)
{
    ui->video_frame->clear();
    switch (index) {
    case R_180P:
        ui->video_frame->addItems({"15fps"});
        m_video_res = R_180P;
        break;
    case R_360P:
        ui->video_frame->addItems({"15fps", "30fps"});
        m_video_res = R_360P;
        break;
    case R_720P:
        ui->video_frame->addItems({"15fps", "30fps"});
        m_video_res = R_720P;
        break;
    case R_1080P:
        ui->video_frame->addItems({"15fps", "30fps", "60fps"});
        m_video_res = R_1080P;
        break;
    }
    qDebug() << __FUNCTION__ << index;
}

void XYSettings::onVideoFrameChanged(int index)
{
    qDebug() << __FUNCTION__ << index;
    switch (index) {
    case FPS_15:
        m_video_fps = FPS_15;
        break;
    case FPS_30:
        m_video_fps = FPS_30;
        break;
    }
}

void XYSettings::onVideoSettingConfirmClicked()
{
    qDebug() << __FUNCTION__ << " m_video_fps " << m_video_fps << " m_video_res " << m_video_res;
    SDKVideoProfile profile;
    profile.maxResPvrx = VideoResolution720P;
    profile.maxFpsPvrx = VideoFrameRate1500;
    switch (m_video_fps) {
    case FPS_15:
        profile.maxFpsPvtx = VideoFrameRate1500;
        break;
    case FPS_30:
        profile.maxFpsPvtx = VideoFrameRate3000;
        break;
    default:
        profile.maxFpsPvtx = VideoFrameRate1500;
    }
    switch (m_video_res) {
    case R_180P:
        profile.maxResPvtx = VideoResolution180P;
        break;
    case R_360P:
        profile.maxResPvtx = VideoResolution360P;
        break;
    case R_720P:
        profile.maxResPvtx = VideoResolution720P;
        break;
    case R_1080P:
        profile.maxResPvtx = VideoResolution1080P;
        break;
    }
    NemoSdkAdaptor::getInstance()->setVideoProfile(profile);
    close();
}

void XYSettings::onDeviceChanged(const SDKMediaDevType& devType) {
    switch (devType) {
    case SDKMediaDevType_Camera:
        getVideoDevices();
        if (!NemoSdkAdaptor::getInstance()->isCameraClosed()) {
            NemoSdkAdaptor::getInstance()->startCamera();
        }
        break;
    case SDKMediaDevType_Microphone:
    case SDKMediaDevType_Speaker:
        getAudioDevices();
        break;
    }
}

void XYSettings::initVirtualBg()
{
    _bgPath.push_back("");

    QDir imgDir(QCoreApplication::applicationDirPath() + "/virtual_bg");
    if (imgDir.exists()) {
        QStringList filters = {"*.jpg", "*.png", "*.bmp"};
        QFileInfoList files = imgDir.entryInfoList(filters);
        for (int i = 0; i < files.size(); ++i) {
            QFileInfo fileInfo = files.at(i);
            qDebug() << fileInfo.path() << fileInfo.fileName();

            QString fullPath = QCoreApplication::applicationDirPath() + "/virtual_bg/" + fileInfo.fileName();
            if (!fullPath.endsWith("jpg"))
            {
                QString jpgPath = converImage2ipg(fullPath);
                imgDir.remove(fullPath);
                _bgPath.push_back(jpgPath);
            }
            if (fullPath.endsWith("jpg")) {
                _bgPath.push_back(fullPath);
            }
        }
    }

    QButtonGroup* _virtualBgBtGroup = new QButtonGroup(this);
    _virtualBgBtGroup->setExclusive(true);
    for (int i = 0; i < _bgPath.size(); ++i) {
        QPushButton* button = new QPushButton(this);
        button->setCheckable(true);
        button->setStyleSheet("QPushButton{color: blue;border-radius: 3px;"
                              "width: 112px;height: 84px;}"
                              "QPushButton:checked{color: blue;border-radius: 3px;border: 1px solid blue;}");
        if (i == 0) {
            button->setChecked(true);
            button->setIcon(QIcon(":/res/beauty/NoEffect_normal.png"));
        } else {
            button->setIcon(QIcon(_bgPath.at(i)));
        }
        button->setIconSize(QSize(112, 84));
        _virtualBgBtGroup->addButton(button, i);
        ui->grid_virtual_bg->addWidget(button, i / COLUMN_VIRTUAL_BG, i % COLUMN_VIRTUAL_BG);
    }
    ui->grid_virtual_bg->setRowStretch(0, 1);
    ui->grid_virtual_bg->setRowStretch(0, COLUMN_VIRTUAL_BG + 1);

    connect(_virtualBgBtGroup, SIGNAL(buttonClicked(int)), this, SLOT(onVirtualBgButtonsChecked(int)));
}

QString XYSettings::converImage2ipg(const QString &imgPath)
{
    QImage image(imgPath);
    QString jpgPaht = imgPath;
    jpgPaht.replace("png", "jpg");
    image.save(jpgPaht, "JPG");
    return jpgPaht;
}

void XYSettings::initBeauty()
{
    _beautyBtGroup = new QButtonGroup(this);
    _beautyBtGroup->setExclusive(true);
    for (int i = 0; i <= SDKBeautyStyle::BeautyStyle_Blush; ++i) {
        QToolButton* button = new QToolButton(this);
        button->setIconSize(QSize(50, 50));
        button->setCheckable(true);
        button->setStyleSheet("QToolButton:checked{color: blue;padding: 3px;border-radius: 3px;border: 1px solid blue;qproperty-iconSize:40px 40px;}");
        button->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        ui->grid_beauty->addWidget(button, 0, i);
        switch (i) {
        case SDKBeautyStyle::BeautyStyle_None:
            button->setText("无");
            button->setChecked(true);
            button->setIcon(QIcon(":/res/beauty/NoEffect_normal.png"));
            break;
        case SDKBeautyStyle::BeautyStyle_Smooth:
            button->setText("磨皮");
            button->setIcon(QIcon(":/res/beauty/SmoothUnChecked.png"));
            break;
        case SDKBeautyStyle::BeautyStyle_Whiten:
            button->setText("美白");
            button->setIcon(QIcon(":/res/beauty/WhitenUnChecked.png"));
            break;
        case SDKBeautyStyle::BeautyStyle_Blush:
            button->setText("红润");
            button->setIcon(QIcon(":/res/beauty/BlushUnChecked.png"));
            break;
        }
        _beautyBtGroup->addButton(button, i);
        if (i != 0) {
            beautySliders[i - 1] = createSlidbar(Tab_Beauty);
        }
    }
    ui->grid_beauty->setRowStretch(1, 1);
    ui->grid_beauty->setColumnStretch(SDKBeautyStyle::BeautyStyle_Blush + 1, 1);
    connect(_beautyBtGroup, SIGNAL(buttonClicked(int)), this, SLOT(onBeautyButtonsChecked(int)));
}

void XYSettings::initFilter()
{
    _filterPath.push_back({"无", ""});
    _filterPath.push_back({"简洁", QCoreApplication::applicationDirPath() + "/filter/01Concise.png"});
    _filterPath.push_back({"自然", QCoreApplication::applicationDirPath() + "/filter/02Natural.png"});
    _filterPath.push_back({"精致", QCoreApplication::applicationDirPath() + "/filter/03Exquisite.png"});
    _filterPath.push_back({"宁静", QCoreApplication::applicationDirPath() + "/filter/04Peaceful.png"});
    _filterPath.push_back({"柔和", QCoreApplication::applicationDirPath() + "/filter/05Soft.png"});
    _filterPath.push_back({"轻奢", QCoreApplication::applicationDirPath() + "/filter/06LightLuxury.png"});
    _filterPath.push_back({"青春", QCoreApplication::applicationDirPath() + "/filter/07Youth.png"});
    _filterPath.push_back({"雨后", QCoreApplication::applicationDirPath() + "/filter/08Rain.png"});
    _filterPath.push_back({"校园", QCoreApplication::applicationDirPath() + "/filter/09Campus.png"});
    _filterPath.push_back({"热情", QCoreApplication::applicationDirPath() + "/filter/10Warmth.png"});
    _filterPath.push_back({"清新", QCoreApplication::applicationDirPath() + "/filter/11Refreshing.png"});
    _filterPath.push_back({"银河", QCoreApplication::applicationDirPath() + "/filter/12Galaxy.png"});
    _filterPath.push_back({"清澈", QCoreApplication::applicationDirPath() + "/filter/13Pure.png"});
    _filterPath.push_back({"清晨", QCoreApplication::applicationDirPath() + "/filter/14Dawn.png"});
    _filterPath.push_back({"梦幻", QCoreApplication::applicationDirPath() + "/filter/15Dream.png"});
    _filterPath.push_back({"高光", QCoreApplication::applicationDirPath() + "/filter/16Highlight.png"});
    _filterPath.push_back({"典雅", QCoreApplication::applicationDirPath() + "/filter/17Elegant.png"});
    _filterPath.push_back({"怀旧", QCoreApplication::applicationDirPath() + "/filter/18Nostalgia.png"});
    _filterPath.push_back({"暗调", QCoreApplication::applicationDirPath() + "/filter/19Dark.png"});
    _filterPath.push_back({"老电影", QCoreApplication::applicationDirPath() + "/filter/20Retro.png"});

    _filterBtGroup = new QButtonGroup(this);
    _filterBtGroup->setExclusive(true);

    for (int i = 0; i < _filterPath.size(); ++i) {
        QToolButton* button = new QToolButton(this);
        button->setCheckable(true);
        if (i == 0) {
            button->setChecked(true);
            button->setIcon(QIcon(":/res/beauty/NoEffect_normal.png"));
        } else {
            button->setIcon(QIcon(_filterPath.at(i).path));
        }
        button->setStyleSheet("QToolButton:checked{color: blue;padding: 3px;border-radius: 3px;border: 1px solid blue;}");
        button->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        button->setIconSize(QSize(60, 60));
        ui->grid_filter->addWidget(button, i / COLUMN_FILTER, i % COLUMN_FILTER);
        button->setText(_filterPath.at(i).name);
        _filterBtGroup->addButton(button, i);

        if (i != 0) {
            filterSliders[i - 1] = createSlidbar(Tab_Filter);
        }
    }
    ui->grid_filter->setRowStretch(0, 1);
    ui->grid_filter->setRowStretch(0, COLUMN_FILTER + 1);

    connect(_filterBtGroup, SIGNAL(buttonClicked(int)), this, SLOT(onFilterButtonsChecked(int)));
}

QSlider* XYSettings::createSlidbar(TabType type)
{
    QSlider* slider = new QSlider(Qt::Horizontal, ui->video_container);
    slider->show();
    slider->setVisible(false);
    slider->setContentsMargins(0, 0, 0, 0);
    slider->setFixedSize(ui->video_container->width() / 2, 20);
    slider->setValue(type == Tab_Filter ? 50 : 25);

    slider->setGeometry((ui->video_container->width() - slider->width()) / 2,
                        ui->video_container->height(),
                        slider->width(), 0);

    slider->setStyleSheet(QString("QSlider::handle:horizontal {"
                                  "background: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 white, stop:1 white);"
                                  "border: 1px solid #5c5c5c;"
                                  "width: 14px;"
                                  "margin: -7px 0;"
                                  "border-radius: 7px;}"
                              ));

    connect(slider, &QSlider::valueChanged, this, &XYSettings::onBeautySliderChanged);
    return slider;
}

void XYSettings::onVirtualBgButtonsChecked(int id)
{
    if (id == _virtualBgCheckId) return;
    _virtualBgCheckId = id;
    if (id == 0) {
        NemoSdkAdaptor::getInstance()->setVirtualBgMode(SDKVirtualBgMode::VirtualBgMode_None);
    } else if (id == 1) {
        NemoSdkAdaptor::getInstance()->setVirtualBgMode(SDKVirtualBgMode::VirtualBgMode_BgBlur);
    } else {
        NemoSdkAdaptor::getInstance()->setVirtualBgMode(SDKVirtualBgMode::VirtualBgMode_BgImage);
        NemoSdkAdaptor::getInstance()->setVirtualBgImage(_bgPath.at(id).toStdString().c_str());
    }
}

void XYSettings::onBeautyButtonsChecked(int id)
{
    if (id == _beautyCheckId) return;
    if (id != 0) beautySliders[id - 1]->setVisible(true);
    if (_beautyCheckId != 0) beautySliders[_beautyCheckId - 1]->setVisible(false);
    _beautyCheckId = id;
    if (id == BeautyStyle_None)
    {
        NemoSdkAdaptor::getInstance()->setVideoBeautyEffect(BeautyStyle_None, 0);
    }
    else
    {
        NemoSdkAdaptor::getInstance()->setVideoBeautyEffect(SDKBeautyStyle(id), beautySliders[id - 1]->value());
    }
}

void XYSettings::onFilterButtonsChecked(int id)
{
    if (id == _filterCheckId) return;
    if (id != 0) filterSliders[id - 1]->setVisible(true);
    if (_filterCheckId != 0) filterSliders[_filterCheckId - 1]->setVisible(false);
    _filterCheckId = id;
    NemoSdkAdaptor::getInstance()->setVideoFilterEffect(SDKFilterStyle(id), filterSliders[id - 1]->value());
}

void XYSettings::onBeautySliderChanged(int value)
{
    switch (ui->tabWidget->currentIndex()) {
    case Tab_VirtualBg:
        break;
    case Tab_Beauty:
        NemoSdkAdaptor::getInstance()->setVideoBeautyEffect(SDKBeautyStyle(_beautyBtGroup->checkedId()), value);
        break;
    case Tab_Filter:
        NemoSdkAdaptor::getInstance()->setVideoFilterEffect(SDKFilterStyle(_filterBtGroup->checkedId()), value);
        break;
    }
}

void XYSettings::onBeautyTabClicked(int index)
{
    switch (index) {
    case Tab_VirtualBg:
        for (int i = BeautyStyle_None; i < BeautyStyle_Blush; ++i) {
            beautySliders[i]->setVisible(false);
        }
        for (int i = FilterStyle_None; i < FilterStyle_Retro; ++i) {
            filterSliders[i]->setVisible(false);
        }
        break;
    case Tab_Beauty:
        if (_beautyCheckId != 0) beautySliders[_beautyCheckId - 1]->setVisible(true);
        for (int i = FilterStyle_None; i < FilterStyle_Retro; ++i) {
            filterSliders[i]->setVisible(false);
        }
        break;
    case Tab_Filter:
        if (_filterCheckId != 0) filterSliders[_filterCheckId - 1]->setVisible(true);
        for (int i = BeautyStyle_None; i < BeautyStyle_Blush; ++i) {
            beautySliders[i]->setVisible(false);
        }
        break;
    }
}
