﻿#ifndef XYSETTINGS_H
#define XYSETTINGS_H

#include <QDialog>
#include <QMouseEvent>
#include <QSettings>
#include <QButtonGroup>
#include <QSlider>
#include "meeting/OpenGLDisplay.h"
#include "NemoSdkAdaptor.h"
#include "meeting/FrameLayout.h"

namespace Ui {
class XYSettings;
}

class XYSettings : public QDialog
{
    Q_OBJECT

public:
    explicit XYSettings(QDialog *parent = nullptr, const char *newParameter = "[1-9][0-9]+$", bool incall = false);
    ~XYSettings();
    enum SettingType { COMMON, AUDIO, VIDEO, BEAUTY, FEEDBACK };
    enum CheckType { SPEAKER, MIC };
    enum VideoResolution { R_180P, R_360P, R_720P, R_1080P };
    enum VideoFPS { FPS_15, FPS_30 };

public:
    void updateUi();
    void getAudioDevices();
    void getVideoDevices();
    void timerCheckEvent(CheckType checkType, bool isStart);
    void onVideoFrameReady(const QString &dataSourceId, const SDKVideoFrameWrapper& videoFrame);

protected:
    void mouseMoveEvent (QMouseEvent *);
    void mousePressEvent (QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);
    void initVirtualBg();
    void initBeauty();
    void initFilter();
    QString converImage2ipg(const QString& imgPath);

signals:
    void sigServerChanged() const;

public slots:
    void onSaveClicked();
    void onResetClicked();
    void onSliderVolumeInChanged(int value);
    void onSliderVolumeOutChanged(int value);
    void onSettingButtonsChecked(int id);
    void onAudioCheckGroupClicked(int id);
    void onSpeakerSelected();
    void onMicSelected();
    void onCameraSelected();
    void onVideoResolutionChanged(int index);
    void onVideoFrameChanged(int index);
    void onVideoSettingConfirmClicked();
    void onDeviceChanged(const SDKMediaDevType& devType);
    void onVirtualBgButtonsChecked(int id);
    void onBeautyButtonsChecked(int id);
    void onFilterButtonsChecked(int id);
    void onBeautySliderChanged(int value);
    void onBeautyTabClicked(int index);
    void onLoguploadResponse(const SDKRestApiResult& result);

private:
    Ui::XYSettings *ui;
    QPoint m_Position;
    bool isPressed;
    QSettings* configs;
    QString lastServerConfig; // 上次的server设置
    QString lastClientIdConfig; // 上次的tClientId设置

    QButtonGroup* toolBar;
    QTimer* speakerTimer; // 扬声器检测timer
    QTimer* micTimer;     // 麦克风检测timer
    bool startMicCheck = false;
    bool startSpeakerCheck = false;
    OpenGLDisplay* openGL;
    OpenGLDisplay* beautyOpenGL;

    // 视频发送帧率
    VideoResolution m_video_res = R_720P;
    VideoFPS m_video_fps = FPS_15;

    // 虚拟背景
    bool _needCloseCamera = true;
    FrameLayout* fl = nullptr;
    QSlider *beautySliders[BeautyStyle_Blush];
    QSlider *filterSliders[FilterStyle_Retro];
    QButtonGroup* _beautyBtGroup = nullptr;
    QButtonGroup* _filterBtGroup = nullptr;
    enum TabType {Tab_VirtualBg, Tab_Beauty, Tab_Filter};
    QSlider* createSlidbar(TabType type);
    int _beautyCheckId = 0;
    int _virtualBgCheckId = 0;
    int _filterCheckId = 0;
    QList<QString> _bgPath;
    struct FilterPath {
        QString name;
        QString path;
    };
    QList<FilterPath> _filterPath; // just for demo convenience
    const int COLUMN_VIRTUAL_BG = 4;
    const int COLUMN_FILTER = 6;
    bool bInCall;
};

#endif // XYSETTINGS_H
