﻿#include "devicemanager.h"
#include "ui_devicemanager.h"

#include <QDebug>
#include <QListView>

struct MediaDevInfo {
    std::string devId;
    std::string devName;
    bool isSelected;
};
Q_DECLARE_METATYPE(MediaDevInfo);

DeviceManager::DeviceManager(QDialog *parent) : QDialog(parent),
    ui(new Ui::DeviceManager), isPressed(false)
{
    ui->setupUi(this);
    this->setWindowFlags(Qt::FramelessWindowHint);

    QFile file(":/stylesheet/style_devices_settings.qss");
    if (file.open(QFile::ReadOnly)) {
        this->setStyleSheet(tr(file.readAll()));
        file.close();
    }


    ui->cb_share_audio->setVisible(false);

    const XYArray<SDKMediaDevInfo> cameraDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Camera);
    qDebug() << "size: " << cameraDeviceList.length();
    if (cameraDeviceList.length() > 0) {
        for (int i = 0; i < cameraDeviceList.length(); i++) {
            if (!QString(cameraDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = cameraDeviceList[i].devId.str();
                mediaInfo.devName = cameraDeviceList[i].devName.str();
                mediaInfo.isSelected = cameraDeviceList[i].isSelected;
                ui->cbox_camera->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected) {
                    ui->cbox_camera->setCurrentText(mediaInfo.devName.c_str());
                }
            }
        }
    }

    const XYArray<SDKMediaDevInfo> micDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Microphone);
    if (micDeviceList.length() > 0) {
        for (int i = 0; i < micDeviceList.length(); i++) {
            if (!QString(micDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = micDeviceList[i].devId.str();
                mediaInfo.devName = micDeviceList[i].devName.str();
                mediaInfo.isSelected = micDeviceList[i].isSelected;
                ui->cbox_mic->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected) {
                    ui->cbox_mic->setCurrentText(mediaInfo.devName.c_str());
                }
            }
        }
    }

    const XYArray<SDKMediaDevInfo> speakerDeviceList = NemoSdkAdaptor::getInstance()->getDevList(SDKMediaDevType_Speaker);
    if (speakerDeviceList.length() > 0) {
        for (int i = 0; i < speakerDeviceList.length(); i++) {
            if (!QString(speakerDeviceList[i].devName.str()).isEmpty()) {
                MediaDevInfo mediaInfo;
                mediaInfo.devId = speakerDeviceList[i].devId.str();
                mediaInfo.devName = speakerDeviceList[i].devName.str();
                mediaInfo.isSelected = speakerDeviceList[i].isSelected;
                ui->cbox_speaker->addItem(mediaInfo.devName.c_str(), QVariant::fromValue(mediaInfo));
                if (mediaInfo.isSelected) {
                    ui->cbox_speaker->setCurrentText(mediaInfo.devName.c_str());
                }
            }
        }
    }

    /*connect(ui->cbox_mic, QOverload<int>::of(&QComboBox::currentIndexChanged), this, [=](int index) {
        qDebug() << "cbox_mic currentIndexChanged: " << index;
    });*/

    connect(ui->pb_confirm, &QPushButton::clicked, this, [=]() {
        qDebug() << "DM pb_confirm";
        QVariant micData =ui->cbox_mic->currentData(Qt::UserRole);
        if (micData.isValid()) {
            MediaDevInfo micInfo = micData.value<MediaDevInfo>();
            if (!micInfo.isSelected) {
                NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Microphone, micInfo.devId.c_str());
                qDebug() << "DM cbo chooseDev micInfo: " << micInfo.devName.c_str() << ", id: " << micInfo.devId.c_str();
            }
        }
        QVariant cameraData =ui->cbox_camera->currentData(Qt::UserRole);
        if (cameraData.isValid()) {
            MediaDevInfo cameraInfo = cameraData.value<MediaDevInfo>();
            if (!cameraInfo.isSelected) {
                NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Camera, cameraInfo.devId.c_str());
                qDebug() << "DM cbo chooseDev cameraInfo: " << cameraInfo.devName.c_str() << ", id: " << cameraInfo.devId.c_str();
            }
        }
        QVariant speakerData =ui->cbox_speaker->currentData(Qt::UserRole);
        if (speakerData.isValid()) {
            MediaDevInfo speakerInfo = speakerData.value<MediaDevInfo>();
            if (!speakerInfo.isSelected) {
                NemoSdkAdaptor::getInstance()->chooseDev(SDKMediaDevType_Speaker, speakerInfo.devId.c_str());
                qDebug() << "DM cbo chooseDev speaker: " << speakerInfo.devName.c_str() << ", id: " << speakerInfo.devId.c_str();
            }
        }
        this->close();
    });
    connect(ui->bt_close, &QToolButton::clicked, this, &QToolButton::close);
    connect(ui->pb_cancel, &QPushButton::clicked, this, &QPushButton::close);
    connect(ui->slider_speaker, &QSlider::valueChanged, this, &DeviceManager::onSpeakerValueChanged);

    int volume = NemoSdkAdaptor::getInstance()->getSpeakerVolume();
    qDebug() << "DM current speaker volume: " << volume;
    ui->slider_speaker->setValue(volume);
}

DeviceManager::~DeviceManager()
{
    delete ui;
}

void DeviceManager::mouseMoveEvent (QMouseEvent * event) {
    if (isPressed) {
        QPoint movePos = event->globalPos() - m_Position;
        move(movePos);
        event->accept();
    }
}

void DeviceManager::mousePressEvent (QMouseEvent * event) {
    if (event->button() == Qt::LeftButton) {
        m_Position = event->pos();
        int titleWidth = ui->topbar->height();
        if (m_Position.y() < titleWidth) {
            isPressed = true;
        }
    }
}
void DeviceManager::mouseReleaseEvent(QMouseEvent *) {
    isPressed = false;
}

void DeviceManager::onSpeakerValueChanged(int value)
{
    NemoSdkAdaptor::getInstance()->setSpeakerVolume(value);
}
