﻿#include "MeetingNotifyWidget.h"
#include "ui_meetingnotifywidget.h"

#include <QFile>

//#include "src/relevance/uidefine.h"
MeetingNotifyWidget::MeetingNotifyWidget(QWidget *parent) :
	QWidget(parent),
	ui(new Ui::MeetingNotifyWidget)
{
	ui->setupUi(this);

	QFile file(":/stylesheet/style_meeting_notification.qss");
	if (file.open(QFile::ReadOnly)) {
		this->setStyleSheet(tr(file.readAll()));
		file.close();
	}
	ui->label_zimu->hide();      // 字幕消息
	ui->label_chairman->hide();  // 主会场
	ui->label_net_tips->hide();  // 网络状况
	ui->lable_outin->hide();     // 出入会通知
	ui->bt_outin_hide->hide();   // 隐藏

	connect(ui->bt_next_page, &QToolButton::clicked, this, [=]() {
		sigNextPageClicked();
	});
	connect(ui->bt_pre_page, &QToolButton::clicked, this, [=]() {
		sigPrePageClicked();
	});
}

MeetingNotifyWidget::~MeetingNotifyWidget()
{
	delete ui;
}

void MeetingNotifyWidget::showZimu(const QString &content, const QString &color, const QString &fontColor,
	MeetingNotifyWidget::Position position, bool isScroll)
{
	ui->label_zimu->setText(content);
	ui->label_zimu->setStyleSheet(QString("QLabel{background-color: %1;font: 18pt \"Microsoft YaHei\";color: %2}")
		.arg(color).arg(fontColor));
	// TODO: position & isScroll
    ui->bt_pre_page->setVisible(false);
    ui->bt_next_page->setVisible(false);
}

void MeetingNotifyWidget::onNotificationChanged(NotifyMode mode, bool visiable, const QString& tips)
{
	switch (mode) {
	case Zimu:
		if (ui->label_zimu->isVisible() != visiable)
			ui->label_zimu->setVisible(visiable);      // 字幕消息
		break;
	case PrePage:
        if (ui->bt_pre_page->isVisible() != visiable) {
			ui->bt_pre_page->setVisible(visiable);     // 上一页
        }
        if (visiable) raise();
		break;
	case NextPage:
		if (ui->bt_next_page->isVisible() != visiable)
        {
            ui->bt_next_page->setVisible(visiable);    // 下一页
        }
        if (visiable) raise();
		break;
	case OutIn:
		if (ui->bt_outin_hide->isVisible() != visiable) {
			ui->lable_outin->setVisible(visiable);     // 出入会通知
			ui->bt_outin_hide->setVisible(visiable);   // 隐藏
		}
		break;
	case Chairman:
		if (ui->label_chairman->isVisible() != visiable)
			ui->label_chairman->setVisible(visiable);  // 主会场
		break;
	case VideoState:
		if (ui->label_net_tips->isVisible() != visiable) {
			if (!tips.isEmpty()) {
				ui->label_net_tips->setText(tips);
			}
			ui->label_net_tips->setVisible(visiable);
		}
		break;
	}
}

void MeetingNotifyWidget::showOut(const QString &outName)
{
	ui->lable_outin->setText(outName);
}

void MeetingNotifyWidget::showIn(const QString &inName)
{
	ui->lable_outin->setText(inName);
}
