﻿#include "MeetingWindow.h"
#include "ui_meetingwindow.h"

#include <QJsonDocument>
#include <QLineEdit>
#include <QPushButton>
#include <QTimer>
#include <QDesktopServices>
#include <QDesktopWidget>

MeetingWindow::MeetingWindow(bool isflipflag, QWidget *parent) :
    BaseWidget(parent),
    ui(new Ui::MeetingWindow),
    _isPressed(false),
    _hasContent(false),
    _totalPeople(0),
    isflip(isflipflag)
{
    ui->setupUi(this);
    ui->bt_info_show->setVisible(false);
    QDesktopWidget *desktop = QApplication::desktop();
    this->move(QApplication::desktop()->screen()->rect().center() - this->rect().center());


    QFile file(":/stylesheet/style_meeting.qss");
    bool result = file.open(QFile::ReadOnly);
    this->setStyleSheet(tr(file.readAll()));
    file.close();

    ui->bt_info_show->setVisible(false);
    ui->tv_meeting_number->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    ui->bt_info_show->setAttribute(Qt::WA_Hover, true);//开启悬停事件
    ui->bt_info_show->installEventFilter(this);       //安装事件过滤器
    ui->tv_meeting_time->setAttribute(Qt::WA_Hover, true);//开启悬停事件
    ui->tv_meeting_time->installEventFilter(this);       //安装事件过滤器

    initToolbar();
    initBottombar();
    initClickEvent();

    // parse json file
    parseErrorCode();

    adaptor = NemoSdkAdaptor::getInstance();
    qRegisterMetaType<std::vector<SDKVideoStreamInfo>>("std::vector<SDKVideoStreamInfo>");
    qRegisterMetaType<SDKCallStateInfo>("SDKCallStateInfo");
    qRegisterMetaType<SDKRosterInfo>("SDKRosterInfo");
    qRegisterMetaType<SDKVideoFrameWrapper>("SDKVideoFrameWrapper");
    qRegisterMetaType<SDKCallInfo>("SDKCallInfo");
    qRegisterMetaType<SDKStatistics>("SDKStatistics");
    qRegisterMetaType<SDKAudioMeterInfo>("SDKAudioMeterInfo");
    qRegisterMetaType<SDKConfMgmtInfo>("SDKConfMgmtInfo");
    qRegisterMetaType<SDKVideoStatus>("SDKVideoStatus");
    qRegisterMetaType<SDKContentState>("SDKContentState");
    //qRegisterMetaType<SDKRecordResponse>("SDKRecordResponse");
    connect(adaptor, &NemoSdkAdaptor::sigConfStateChanged, this, &MeetingWindow::onConfStateChanged);
    connect(adaptor, &NemoSdkAdaptor::sigConfMemberChanged, this, &MeetingWindow::onConfMemberChanged);
    connect(adaptor, &NemoSdkAdaptor::sigConfMgmtChanged, this, &MeetingWindow::onConfMgmtChanged);
    connect(adaptor, &NemoSdkAdaptor::sigRosterChanged, this, &MeetingWindow::onRosterChanged);
    connect(adaptor, &NemoSdkAdaptor::sigStreamChanged, this, &MeetingWindow::onStreamChanged);
    connect(adaptor, &NemoSdkAdaptor::sigVideoFrameReady, this, &MeetingWindow::onVideoFrameReady);
    connect(adaptor, &NemoSdkAdaptor::sigVideoStateChanged, this, &MeetingWindow::onVideoStateChanged);
    connect(adaptor, &NemoSdkAdaptor::sigNetworkIndicatorLevel, this, &MeetingWindow::onNetworkIndicatorLevel);
    connect(adaptor, &NemoSdkAdaptor::sigKickOut, this, &MeetingWindow::onKickOut);
    connect(adaptor, &NemoSdkAdaptor::sigContentStateChanged, this, &MeetingWindow::onContentStateChagend);
    connect(adaptor, &NemoSdkAdaptor::sigConfHandupCancelled, this, &MeetingWindow::onConfHandupCancelled);
    connect(adaptor, &NemoSdkAdaptor::sigConfHostChanged, this, [=](bool isConfHost) {
        if (_isConfHost) {
            btHost->setChecked(QString::fromLocal8Bit("主持"), iconHostNormal, iconHostHover, true);
        } else {
            btHost->setChecked(QString::fromLocal8Bit("参会者"), iconHostNormal, iconHostHover, true);
        }
    });
    //connect(adaptor, &NemoSdkAdaptor::sigRecordResponse, this, &MeetingWindow::onRecordResponse);
}

void MeetingWindow::onMicEnergyReported(int energy)
{
    volume = energy;
    //qDebug() << "voice energy=" + QString::number(volume);
}

MeetingWindow::~MeetingWindow()
{
    qDebug() << "~MeetingWindow()";
    delete ui;
    delete timer;
    delete _bottomBarTimer;
    if (OpenNetWorkDataShow != nullptr)
    {
        OpenNetWorkDataShow->close();
        OpenNetWorkDataShow->deleteLater();
        OpenNetWorkDataShow = nullptr;
    }
    if (_infoshow != nullptr)
    {
        _infoshow->close();
        _infoshow->deleteLater();
        _infoshow = nullptr;
    }

    if (shareDlg) {
        shareDlg->close();
        delete shareDlg;
        shareDlg = nullptr;
    }
    if (_shareRegWin != nullptr) {
        _shareRegWin->close();
        delete _shareRegWin;
        _shareRegWin = nullptr;
    }
}

void MeetingWindow::initToolbar()
{
    ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_04.png"));
    connect(ui->bt_net_level, &QToolButton::clicked, this, [=]() {
        if (!NetWorkDataShowed)
        {
            if (OpenNetWorkDataShow != nullptr)
                delete OpenNetWorkDataShow;
            OpenNetWorkDataShow = new NetWorkDataShow;
            OpenNetWorkDataShow->setWindowOpacity(0.95);
            OpenNetWorkDataShow->setWindowFlags(Qt::FramelessWindowHint);
            OpenNetWorkDataShow->show();
            NetWorkDataShowed = true;
            connect(OpenNetWorkDataShow, &NetWorkDataShow::closeNetWorkDataShow, this, &MeetingWindow::onCloseNetWorkDataShow);
        }
    });
    videoWidget = new QWidget(this);
    frameLayout = new FrameLayout(this, 0, 0, 0);
    frameLayout->addWidget(videoWidget);
    meetingStateWidget = new MeetingStateWidget(this);
    frameLayout->addWidget(meetingStateWidget);
    notifyWidget = new MeetingNotifyWidget(this);
    frameLayout->addWidget(notifyWidget);

    meetingLayout = new MeetingLayout(this, 0, 0, 0);
    videoWidget->setLayout(meetingLayout);
    ui->video_holder->setLayout(frameLayout);
    _localVideoCell = new VideoCell(this);
    _localInfo.videoSourceId = "LocalPreviewID";
    meetingLayout->addWidget(_localVideoCell);
    timer = new QTimer(this);
    qmeetingTime = QTime(0, 0, 0);
    connect(timer, &QTimer::timeout, this, &MeetingWindow::onTimerEvent);

    meetingStateWidget->activateWindow();
    meetingStateWidget->raise();
}

void MeetingWindow::initBottombar()
{
    btSetting = new MenuButton(QString::fromLocal8Bit("设置"), iconSettingNormal, iconSettingHover, this);
    btShare = new MenuButton(QString::fromLocal8Bit("共享主屏"), iconShareNormal, iconShareHover, this);
    btFullScreen = new MenuButton(QString::fromLocal8Bit("全屏"), iconFullScreenNormal, iconFullScreenHover, this);
    if (_isConfHost) {
        btHost = new MenuButton(QString::fromLocal8Bit("主持"), iconHostNormal, iconHostHover, this);
    } else {
        btHost = new MenuButton(QString::fromLocal8Bit("参会者"), iconHostNormal, iconHostHover, this);
    }
    btLayou = new MenuButton(QString::fromLocal8Bit("切换布局"), iconLayoutNormal, iconLayoutHover, this);
    btRecord = new MenuButton(QString::fromLocal8Bit("开始录制"), iconRecordNormal, iconRecordHover, this);
    btCamera = new MenuButton(QString::fromLocal8Bit("关闭摄像头"), iconCameraNormal, iconCameraHover, this);
    btMic = new MenuButton(QString::fromLocal8Bit("静音"), iconMicNormal, iconMicHover, this);
    btHangup = new MenuButton(QString::fromLocal8Bit("挂断"), iconHangupNormal, iconHangupHover, this);

    //    btFaceLocalFDDetect = new MenuButton(QString::fromLocal8Bit("扫描模式"), iconHangupNormal, iconHangupHover, this);
    //    btFaceLocalFDBadge = new MenuButton(QString::fromLocal8Bit("铭牌模式"), iconHangupNormal, iconHangupHover, this);
    //    btFaceEletronicBadge = new MenuButton(QString::fromLocal8Bit("广播本地电子铭牌"), iconHangupNormal, iconHangupHover, this);
    //btFaceGetInfo = new MenuButton(QString::fromLocal8Bit("GetInfo"), iconHangupNormal, iconHangupHover, this);

    QHBoxLayout* hLayout = new QHBoxLayout;
    hLayout->setContentsMargins(0, 0, 0, 0);
    hLayout->addWidget(btSetting);
    hLayout->addWidget(btFullScreen);
    hLayout->addWidget(btShare);
    hLayout->addWidget(btHost);
    hLayout->addWidget(btLayou);
    hLayout->addWidget(btRecord);
    hLayout->addStretch();
    hLayout->addWidget(btCamera);
    hLayout->addWidget(btMic);
    hLayout->addWidget(btHangup);

    // face test
    //    hLayout->addWidget(btFaceLocalFDDetect);
    //    hLayout->addWidget(btFaceLocalFDBadge);
    //    hLayout->addWidget(btFaceEletronicBadge);
    //    hLayout->addWidget(btFaceGetInfo);

    ui->bottom_bar->setLayout(hLayout);

    ui->bottom_bar_holder->setVisible(false);

    _bottomBarTimer = new QTimer(this);
    connect(_bottomBarTimer, &QTimer::timeout, this, &MeetingWindow::onBottombarTimerEvent);
}
#include <QThread>
void MeetingWindow::initClickEvent()
{
    // 设置
    connect(btSetting, &MenuButton::clicked, this, [=]() {
        XYSettings* xySettings = new XYSettings();
        xySettings->show();
    });

    // 共享
    connect(btShare, &MenuButton::clicked, this, [=]() {
        if (adaptor->isContentSharing()) {
            adaptor->stopContentSharing();
            if (shareDlg) {
                shareDlg->close();
                delete shareDlg;
                shareDlg = nullptr;
            }
            if (_shareRegWin != nullptr) {
                delete _shareRegWin;
                _shareRegWin = nullptr;
            }
            //            btShare->setChecked(QString::fromLocal8Bit("开始共享"), iconShareNormal, iconShareHover, true);
            qDebug() << __FUNCTION__ << __LINE__;
        }
        else {
            if (shareDlg == nullptr) {
                shareDlg = new ShareDialog();
                shareDlg->show();
            }
            connect(shareDlg, &ShareDialog::sigShareScreen, this, [=](ShareDialog::ShareType type, const QString& monitorName) {
                if (type == ShareDialog::ShareType::ShareType_Monitor)
                {
                    SDKContentInfo contentInfo;
                    contentInfo.withAudio = true;
                    adaptor->startContentSharing(monitorName, contentInfo);
                } else {
                    if (_shareRegWin == nullptr) {
                        _shareRegWin = new ShareRegionWidget();
                        _shareRegWin->show();
                    }
                    QRect initRect = _shareRegWin->defauleRegion();
                    SDKContentInfo contentInfo;
                    contentInfo.withAudio = true;
                    contentInfo.region.x = initRect.x();
                    contentInfo.region.y = initRect.y();
                    contentInfo.region.width = initRect.width();
                    contentInfo.region.height = initRect.height();
                    adaptor->startContentSharing(monitorName, contentInfo);

                    connect(_shareRegWin, &ShareRegionWidget::sigShareRegChanged, this, [&](int x, int y, int w, int h) {
                        adaptor->updateContentCaptureRegion(x, y, w, h);
                    });
                }
            });

            connect(shareDlg, &ShareDialog::sigShareDlgClosed, this, [=]() {
                if (shareDlg) {
                    shareDlg->deleteLater();
                    shareDlg = nullptr;
                }
            });
        }
    });

    // 全屏
    connect(btFullScreen, &MenuButton::clicked, this, [=]() {
        qDebug() << " windowState : " << this->windowState();
        if (this->windowState() != Qt::WindowFullScreen) {
            this->showFullScreen();
            btFullScreen->onWindowStateChange(Qt::WindowFullScreen);
            ui->bt_max->setStyleSheet("QToolButton{border: 0px; image: url(:/res/meeting/restore_normal.png);}QToolButton:hover{border: 0px; image: url(:/res/meeting/restore_over.png);}QToolButton:pressed{border: 0px; image: url(:/res/meeting/restore_click.png);}");
        }
        else {
            this->showNormal();
            btFullScreen->onWindowStateChange(Qt::WindowNoState);
            ui->bt_max->setStyleSheet("QToolButton{border: 0px; image: url(:/res/meeting/max_normal.png);}QToolButton:hover{border: 0px; image: url(:/res/meeting/max_over.png);}QToolButton:pressed{border: 0px; image: url(:/res/meeting/max_click.png);}");
        }
    });

    // 切换布局
    connect(btLayou, &MenuButton::clicked, this, &MeetingWindow::onLayoutClicked);

    // 会控
    connect(btHost, &MenuButton::clicked, this, [=]() {
        if (_confMgmtUrl.isEmpty())
        {
            return;
        }
        if (_confHostWindow == nullptr)
        {
            _confHostWindow = new MettingHostWindow(nullptr);
            _confHostWindow->loadUrl(NemoSdkAdaptor::getInstance()->getConfMgmtUrl("9005524137"));
            _confHostWindow->show();
            connect(_confHostWindow, &MettingHostWindow::sigMeetignHostClosed, this, [=]() {
                if (_confHostWindow) {
                    delete _confHostWindow;
                    _confHostWindow = nullptr;
                }
            });
        }
        // QDesktopServices::openUrl(_confMgmtUrl); // 系統瀏覽器打開
    });

    // 录制
    connect(btRecord, &MenuButton::clicked, this, [=](){
        if (!adaptor->isRecordEnable()) return;
        if (adaptor->isRecordStartBySelf()) {
            qDebug() << "stopRecord";
            btRecord->setChecked(QString::fromLocal8Bit("开始录制"), iconRecordNormal, iconRecordHover, true);
            adaptor->stopRecord();
        } else {
            qDebug() << "startRecord";
            btRecord->setChecked(QString::fromLocal8Bit("结束录制"), iconRecordStop, iconRecordStop, true);
            adaptor->startRecord();
        }
    });

    // 摄像头
    connect(btCamera, &MenuButton::clicked, this, [=]() {
        if (adaptor->isCameraClosed()) {
            adaptor->startCamera();
            adaptor->setVideoMute(false);
            _localInfo.videoState = SDKVideoState_Received;
            _localVideoCell->setVidoeInfo(_localInfo);
            btCamera->setChecked(QString::fromLocal8Bit("关闭摄像头"), iconCameraNormal, iconCameraHover, true);
        }
        else {
            adaptor->stopCamera();
            adaptor->setVideoMute(true);
            _localInfo.videoState = SDKVideoState_VideoMute;
            _localVideoCell->setVidoeInfo(_localInfo);
            btCamera->setChecked(QString::fromLocal8Bit("打开摄像头"), iconCameraClicked, iconCameraClicked, false);
        }
    });

    // 麦克风
    connect(btMic, &MenuButton::clicked, this, [=]() {
        if (Muteflag == 0)
        {
            if (adaptor->isMicMuted()) {
                adaptor->setMicMute(false);
                btMic->setChecked(QString::fromLocal8Bit("静音"), iconMicNormal, iconMicHover, true);
            }
            else {
                adaptor->setMicMute(true);
                btMic->setChecked(QString::fromLocal8Bit("取消静音"), iconMicClicked, iconMicClicked, false);
            }
            _localVideoCell->setLocalAudioMute(adaptor->isMicMuted());
        }
        else if (Muteflag == 1)
        {
            adaptor->sendSpeakingRequest();
            Muteflag = 2;
            btMic->setChecked(QString::fromLocal8Bit("取消举手"), iconhanddown, iconhanddownhover, true);
        }
        else if (Muteflag == 2)
        {
            adaptor->cancelSpeakingRequest();
            Muteflag = 1;
            btMic->setChecked(QString::fromLocal8Bit("举手发言"), iconhandup, iconhanduphover, true);
        }
        else if (Muteflag == 3)
        {
            adaptor->sendSpeakingEnd();
            Muteflag = 1;
            btMic->setChecked(QString::fromLocal8Bit("举手发言"), iconhandup, iconhanduphover, true);
        }
    });

    // 挂断
    connect(btHangup, &MenuButton::clicked, this, [=]() {
        if (adaptor->isContentSharing()) {
            adaptor->stopContentSharing();

        }
        adaptor->stopCamera();
        adaptor->dropCall();
        emit sigMeetingFinished("STATE:200");
    });

    // 最小化
    connect(ui->bt_min, &QToolButton::clicked, this, [=]() {
        this->showMinimized();
        if (_infoshow != nullptr)
        {
            _infoshow->hide();
        }
    });

    // 关闭
    connect(ui->bt_close, &QToolButton::clicked, this, [=]() {
        adaptor->stopCamera();
        adaptor->dropCall();
        emit sigMeetingFinished("STATE:200");
    });

    // 最大化
    connect(ui->bt_max, &QToolButton::clicked, this, [=]() {
        if (this->windowState() != Qt::WindowMaximized) {
            this->showMaximized();
            ui->bt_max->setStyleSheet("QToolButton{border: 0px; image: url(:/res/meeting/restore_normal.png);}QToolButton:hover{border: 0px; image: url(:/res/meeting/restore_over.png);}QToolButton:pressed{border: 0px; image: url(:/res/meeting/restore_click.png);}");
        }
        else {
            this->showNormal();
            ui->bt_max->setStyleSheet("QToolButton{border: 0px; image: url(:/res/meeting/max_normal.png);}QToolButton:hover{border: 0px; image: url(:/res/meeting/max_over.png);}QToolButton:pressed{border: 0px; image: url(:/res/meeting/max_click.png);}");
        }
    });

    // 会议密码
    connect(meetingStateWidget, &MeetingStateWidget::sigPasswordConfirm, this, [=](const QString& password) {
        qDebug() << "pwd::: " << password;
        adaptor->makeCall(_number.toStdString(), password.toStdString(), SDKCallMode_AudioVideo, _displayName.toStdString());
    });

    // 来电
    connect(meetingStateWidget, &MeetingStateWidget::sigInviateAccept, this, [=]() {
        qDebug() << "sigInviateAccept";
        acceptCall(true, true);
    });

    connect(meetingStateWidget, &MeetingStateWidget::sigInviateRefuse, this, [=]() {
        qDebug() << "sigInviateRefuse";
        adaptor->stopCamera();
        adaptor->dropCall();
        emit sigMeetingFinished("STATE:200");
    });

    // 去电
    connect(meetingStateWidget, &MeetingStateWidget::sigOutgoingHangup, this, [=]() {
        adaptor->stopCamera();
        adaptor->dropCall();
        emit sigMeetingFinished("STATE:200");
    });

    // 上一页
    connect(notifyWidget, &MeetingNotifyWidget::sigPrePageClicked, this, [=]() {
        if (meetingLayout->currentLayoutMode() == MeetingLayout::Speaker || (meetingLayout->currentLayoutMode() == MeetingLayout::Gallery)) {
            pageIndex--;
            if (pageIndex == 0) {
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::PrePage, false);
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::NextPage, _totalPeople > 1);
            }
            bool updateLayout = false;
            int layoutReqNum = 0;

            layoutReqNum = MeetingLayout::LAYOUT_GALLERT_NUM;
            if (pageIndex == 0) {
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::PrePage, false);
                meetingLayout->setLayoutMode(MeetingLayout::Speaker);
                updateLayout = true;
            } else {
                meetingLayout->setLayoutMode(MeetingLayout::Gallery);
                meetingLayout->setPageModeShowLocalPreview(_uiShowLocalWhenPageMode);
            }
            meetingLayout->setPageModeIndex(pageIndex);
            adaptor->reqAutoLayout(layoutReqNum, _hasContent, "", meetingLayout->currentLayoutMode(), pageIndex, _uiShowLocalWhenPageMode);
            if (updateLayout) {
                meetingLayout->update();
            }
        }
    });

    // 下一页
    connect(notifyWidget, &MeetingNotifyWidget::sigNextPageClicked, this, [=]() {
        if (meetingLayout->currentLayoutMode() == MeetingLayout::Speaker || (meetingLayout->currentLayoutMode() == MeetingLayout::Gallery)) {
            if (isLastPage()) {
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::NextPage, false);
                return;
            }
            pageIndex++;

            if (pageIndex > 0) {
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::PrePage, true);
            }
            meetingLayout->setLayoutMode(MeetingLayout::Gallery);
            int layoutReqNum = 0;
            layoutReqNum = MeetingLayout::LAYOUT_GALLERT_NUM;
            meetingLayout->setPageModeIndex(pageIndex);
            adaptor->reqAutoLayout(layoutReqNum, false, "", meetingLayout->currentLayoutMode(), pageIndex, false);
            meetingLayout->setPageModeShowLocalPreview(_uiShowLocalWhenPageMode);

            if (isLastPage()) {
                notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::NextPage, _uiShowLocalWhenPageMode);
            }
        }
    });

#ifdef BEAUTY_TEST
    connect(btFaceLocalFDDetect, &MenuButton::clicked, this, [=]() {
        adaptor->enableLocalFD(SDKFaceType_Detect, true);
    });

    connect(btFaceLocalFDBadge, &MenuButton::clicked, this, [=]() {
        adaptor->enableLocalFD(SDKFaceType_EletronicBadge, true);
    });

    connect(btFaceEletronicBadge, &MenuButton::clicked, this, [=]() {
        adaptor->enableEletronicBadge(true);
    });

    connect(btFaceGetInfo, &MenuButton::clicked, this, [=]() {
        adaptor->setVirtualBgMode(SDKVirtualBgMode::VirtualBgMode_BgImage);
        adaptor->setVirtualBgImage("C:\\Users\\Thinkpad\\Pictures\\20200412133634_50262.jpg");
    });
#endif

}

void MeetingWindow::makeCall(const QString &meetingNumber, const QString &displayName, const QString &password,
                             bool videoMute, bool audioMute, const QString& meetingId)
{
    qDebug() << "meetingNumber: " << meetingNumber << ", displayName: " << displayName;
    meetingStateWidget->setMeetingState(MeetingStateWidget::Outgoing);
    QByteArray ba = meetingNumber.toLatin1(); // must
    const char*ch = ba.data();
    _isMakecall = true;
    //meetingStateWidget->findChild<QLabel*>("tv_outgoing_name")->setText(meetingNumber);
    if (videoMute) {
        adaptor->stopCamera();
        _localInfo.videoState = SDKVideoState_VideoMute;
        btCamera->setChecked(QString::fromLocal8Bit("打开摄像头"), iconCameraClicked, iconCameraClicked, false);
    }
    else {
        adaptor->startCamera();
        _localInfo.videoState = SDKVideoState_Received;
        btCamera->setChecked(QString::fromLocal8Bit("关闭摄像头"), iconCameraNormal, iconCameraHover, true);
    }
    if (audioMute) {
        btMic->setChecked(QString::fromLocal8Bit("取消静音"), iconMicClicked, iconMicClicked, false);
    }
    else {
        btMic->setChecked(QString::fromLocal8Bit("静音"), iconMicNormal, iconMicHover, true);
    }
    _localInfo.displayName = displayName.toStdString().c_str();
    _localInfo.audioMute = audioMute;
    _localVideoCell->setVidoeInfo(_localInfo, false);
    adaptor->makeCall(meetingNumber.toStdString(), password.toStdString(), SDKCallMode_AudioVideo,
                      displayName.toStdString(), meetingId);
    this->_number = meetingNumber;
    this->_displayName = displayName;
}

void MeetingWindow::acceptCall(bool videoMute, bool audioMute)
{
    adaptor->acceptCall();
    if (videoMute) {
        adaptor->stopCamera();
        _localInfo.videoState = SDKVideoState_VideoMute;
        btCamera->setChecked(QString::fromLocal8Bit("打开摄像头"), iconCameraClicked, iconCameraClicked, false);
    }
    else {
        adaptor->startCamera();
        _localInfo.videoState = SDKVideoState_Received;
        btCamera->resetState(QString::fromLocal8Bit("关闭摄像头"), iconCameraNormal, iconCameraHover, true);
    }
    _localInfo.audioMute = audioMute;
    if (audioMute) {
        btMic->setChecked(QString::fromLocal8Bit("取消静音"), iconMicClicked, iconMicClicked, false);
    }
    else {
        btMic->setChecked(QString::fromLocal8Bit("静音"), iconMicNormal, iconMicHover, true);
    }
    _localInfo.displayName = _displayName.toStdString().c_str();
    _localVideoCell->setVidoeInfo(_localInfo, false);
}

void MeetingWindow::onMeetingInvited(const QString& displayName, const SDKCallInfo &callInfo, bool videoMute, bool audioMute)
{
    qDebug() << "onMeetingInvited: " << callInfo.remoteNumber.str();
    _inviteCallInfo = callInfo;
    QByteArray base64Name = callInfo.remoteName.str();
    QByteArray inviteName = QByteArray::fromBase64(base64Name);
    QString str = QString(inviteName) + "邀请您视频通话";
    meetingStateWidget->setMeetingState(MeetingStateWidget::Incoming);
    meetingStateWidget->findChild<QLabel*>("tv_invite_name")->setText(str);
    this->_displayName = displayName;
    this->_number = callInfo.remoteNumber.str();
}

bool MeetingWindow::isLastPage()
{
    int totalNum = _uiShowLocalWhenPageMode ? (_totalPeople + 1) : _totalPeople;
    return pageIndex == totalNum / MeetingLayout::LAYOUT_GALLERT_NUM + (totalNum % MeetingLayout::LAYOUT_GALLERT_NUM > 0 ? 1 : 0);
}

void MeetingWindow::setUIShowLocalWhenPageMode(bool uiShowLocalWhenPageMode)
{
    _uiShowLocalWhenPageMode = uiShowLocalWhenPageMode;
}

void MeetingWindow::leaveEvent(QEvent *)
{
    if (!_bottomBarTimer->isActive()) {
        _bottomBarTimer->start(3000);
    }
}

void MeetingWindow::enterEvent(QEvent *)
{
    if (_bottomBarTimer->isActive()) {
        _bottomBarTimer->stop();
    }
    if (!ui->bottom_bar->isVisible()) {
        ui->bottom_bar->setVisible(true);
    }
}

void MeetingWindow::onTimerEvent()
{
    qmeetingTime = qmeetingTime.addSecs(1);
    ui->tv_meeting_time->setText(qmeetingTime.toString("hh:mm:ss"));
    if ((!adaptor->isMicMuted()) && (Muteflag == 0))
    {
        if (volume < 60) {
            btMic->resetState(QString::fromLocal8Bit("静音"), iconMicLev1, iconMicHover);
        } else if ((volume >= 60) && (volume < 70)) {
            btMic->resetState(QString::fromLocal8Bit("静音"), iconMicLev2, iconMicHover);
        }  else {
            btMic->resetState(QString::fromLocal8Bit("静音"), iconMicLev3, iconMicHover);
        }
    }  else if ((adaptor->isMicMuted()) && (Muteflag == 0)) {
        btMic->setChecked(QString::fromLocal8Bit(" 取消静音 "), ":/res/meeting/open_mic_click.png", ":/res/meeting/open_mic_click.png", false);
    }
    bool CamFlag = adaptor->isCameraClosed();
    if (CamFlag == false) {
        btCamera->resetState(QString::fromLocal8Bit(" 关闭摄像头 "), ":/res/meeting/camera_normal.png", ":/res/meeting/camera_hover.png");
    } else if (CamFlag == true) {
        btCamera->setChecked(QString::fromLocal8Bit(" 打开摄像头 "), ":/res/meeting/open_camera_click.png", ":/res/meeting/open_camera_click.png", false);
    }
    if (infoShowNum != 0) {
        infoShowNum--;
    } else {
        if (_infoshow != nullptr) {
            _infoshow->close();
            delete _infoshow;
            _infoshow = nullptr;
        }
    }
}

void MeetingWindow::onBottombarTimerEvent()
{
    if (ui->bottom_bar->isVisible())
        ui->bottom_bar->setVisible(false);
}

void MeetingWindow::singleShotSlot()
{
    emit sigMeetingFinished("call_disconnected");
}

void MeetingWindow::onConfStateChanged(SDKCallStateInfo callStateInfo)
{
    qDebug() << "onConfStateChanged : " << callStateInfo.callState << "disconnectReason" << callStateInfo.disconnectReason.str()
             << "" << callStateInfo.sdkError.str();
    const char*  ch;
    QByteArray ba;
    switch (callStateInfo.callState) {
    case SDKCallState_Connected:
        ui->bottom_bar_holder->setVisible(true);
        qmeetingTime.setHMS(0, 0, 0);
        timer->start(1000);
        ui->tv_meeting_number->setText(callStateInfo.meetingNumber.str());
        meetingStateWidget->setMeetingState(MeetingStateWidget::NONE);
        _localVideoCell->setVidoeInfo(_localInfo);
        adaptor->setMicMute(_localInfo.audioMute);
        adaptor->setVideoMute(_localInfo.videoState == SDKVideoState_VideoMute);

        qDebug() << "MeetingWindow::onConfStateChanged: " << adaptor->getConfMgmtUrl(callStateInfo.meetingNumber.str());
        break;
    case SDKCallState_Connecting:
        break;
    case SDKCallState_Disconnected:
        if (!adaptor->isCameraClosed()) {
            adaptor->stopCamera();
        }
        if (_bottomBarTimer->isActive())
            _bottomBarTimer->stop();
        if (ui->bottom_bar->isVisible())
            ui->bottom_bar->setVisible(false);
        if (_bottomBarTimer->isActive())
            _bottomBarTimer->stop();
        if (QString(callStateInfo.sdkError.str()) == SDKError_WrongPassword ||
                QString("InvalidConfPasswd").compare(callStateInfo.disconnectReason.str()) == 0) {
            meetingStateWidget->setMeetingState(MeetingStateWidget::Password);
        } else {
            meetingStateWidget->setMeetingState(MeetingStateWidget::Tips);

            QString tipsStr = QString::fromLocal8Bit("通话已结束");
            if (callStateInfo.sdkError.str() == SDKError_NoError) {
                QString codeInfo = NemoSdkAdaptor::getInstance()->error_info(callStateInfo.disconnectReason.str());
                if (!codeInfo.isEmpty()) tipsStr = codeInfo;
            } else {
                QString errInfo = NemoSdkAdaptor::getInstance()->error_info(callStateInfo.sdkError.str());
                if (!errInfo.isEmpty()) tipsStr = errInfo;
            }
            meetingStateWidget->findChild<QLabel*>("tv_toast_tips")->setText(tipsStr);
            QTimer::singleShot(3000, this, &MeetingWindow::singleShotSlot);
        }
        break;
    case SDKCallState_Disconnecting:
        break;
    }
}

void MeetingWindow::onConfMgmtChanged(const SDKConfMgmtInfo &confMgmtInfo)
{
    qDebug() << __func__ << confMgmtInfo.disableRecord;
    _chirmanUri = confMgmtInfo.chirmanUri.str();
    _confMgmtUrl = confMgmtInfo.confMgmtUrl.str();
    _confMute = confMgmtInfo.muteMic.str();

    bool isChairmanMode = !_chirmanUri.isEmpty();
    notifyWidget->onNotificationChanged(MeetingNotifyWidget::Chairman, isChairmanMode);

    isContentShareEnable = !confMgmtInfo.disableContent;
    btShare->setEnabled(isContentShareEnable);
    btRecord->setEnabled(adaptor->isRecordEnable());
    if (confMgmtInfo.disableContent) {
        btShare->resetState(QString::fromLocal8Bit("开始共享"), iconShareDisable, iconShareDisable, true);
    }
    else {
        btShare->resetState(QString::fromLocal8Bit("开始共享"), iconShareNormal, iconShareHover, false);
    }

    if (adaptor->isRecordEnable()) {
        btRecord->resetState(QString::fromLocal8Bit("开始录制"), iconRecordNormal, iconRecordHover, false);
    } else {
        btRecord->resetState(QString::fromLocal8Bit("开始录制"), iconRecordStartDisable, iconRecordStartDisable, true);
    }

    if (confMgmtInfo.disableMute == true) {
        if ((_confMute == "mute") || (_confMute == "forcemute")) {
            Muteflag = 1;
            btMic->setChecked(QString::fromLocal8Bit("举手发言"), iconhandup, iconhanduphover, true);
        } else if (_confMute == "unmute") {
            Muteflag = 3;
            btMic->setChecked(QString::fromLocal8Bit("结束发言"), iconendspeak, iconendspeakhover, true);
        }  else {
            Muteflag = 1;
            btMic->setChecked(QString::fromLocal8Bit("举手发言"), iconhandup, iconhanduphover, true);
        }
    } else {
        Muteflag = 0;
        if ((_confMute == "mute") || (_confMute == "forcemute")) {
            adaptor->setMicMute(true);
            btMic->setChecked(QString::fromLocal8Bit("取消静音"), iconMicClicked, iconMicClicked, false);
        } else if (_confMute == "unmute") {
            adaptor->setMicMute(false);
            btMic->setChecked(QString::fromLocal8Bit("静音"), iconMicNormal, iconMicHover, true);
        }
    }
}

/**
 * @brief MeetingWindow::onConfMemberChanged
 *
 * onConfMemberChanged: 参会人员信息变化回调
 * reqAutoLayout: 请求视频流, 参会人员信息变化时, 在此处根据业务场景请求视频流
 *
 * Speaker: 演讲者模式 (1 + N)
 * Gallery: 画廊模式 (对称屏)
 * Chairman: 主会场模式 (1 + 1) -> 1(big: chairman主会场) + 1(local)
 * Focus: 专注模式 (1 + 1) -> 1 (big: conent) + 1 (as: 讲话者)
 * ContentOnly: 内容模式 (1) -> 只显示content
 * ContentAndAs: content & as 对称显示
 */
void MeetingWindow::onConfMemberChanged(int num, bool hasContent, QString chirmanUri)
{
    if (pageIndex == 0 && num > 2) {
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::NextPage, true);
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::PrePage, false);
    }
    if (num <= 1) {
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::NextPage, false);
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::NotifyMode::PrePage, false);
    }
    btHost->showLabel(num);
    _totalPeople = num;

    _hasContent = hasContent;

    bool updateLayout = false;
    qDebug() << "onConfMemberChanged : " << num << ", hasContent : " << hasContent << ", chairmanUri : " << chirmanUri;

    if (num > 0) {
        if (pageIndex > 0 && meetingLayout->currentLayoutMode() == MeetingLayout::Gallery) {
            return;
        }
        if (!chirmanUri.isEmpty()) {
            meetingLayout->setLayoutMode(MeetingLayout::Chairman);
            updateLayout = true;
        }
        if (chirmanUri.isEmpty() && meetingLayout->currentLayoutMode() == MeetingLayout::Chairman) {
            meetingLayout->setLayoutMode(MeetingLayout::Speaker);
            updateLayout = true;
        }
        if (hasContent && meetingLayout->currentLayoutMode() == MeetingLayout::Gallery) {
            meetingLayout->setLayoutMode(MeetingLayout::Speaker);
            updateLayout = true;
        }
        int layoutReqNum = 0;
        switch (meetingLayout->currentLayoutMode()) {
        case MeetingLayout::Speaker:
            layoutReqNum = num > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : num;
            break;
        case MeetingLayout::Gallery:
            layoutReqNum = MeetingLayout::LAYOUT_GALLERT_NUM;
            break;
        case MeetingLayout::Chairman:
        case MeetingLayout::Focus:
        case MeetingLayout::ContentAndAs:
            layoutReqNum = hasContent ? 2 : 1;
            break;
        case MeetingLayout::Locked:
        case MeetingLayout::ContentOnly:
            layoutReqNum = 1;
            break;
        case MeetingLayout::ContentShare:
            layoutReqNum = num > 3 ? 3 : num;
            break;
        }
        adaptor->reqAutoLayout(layoutReqNum, hasContent, chirmanUri.toStdString(), meetingLayout->currentLayoutMode());
        if (updateLayout) {
            meetingLayout->update();
        }
        if (meetingLayout->currentLayoutMode() == MeetingLayout::Chairman || _totalPeople < 2 ||
                meetingLayout->currentLayoutMode() == MeetingLayout::ContentShare) {
            btLayou->resetState(QString::fromLocal8Bit("切换布局"), iconLayoutDisable, iconLayoutDisable, true);
        } else {
            btLayou->resetState(QString::fromLocal8Bit("切换布局"), iconLayoutNormal, iconLayoutHover, false);
        }
    }
}

/**
 * @brief MeetingWindow::onLayoutClicked
 *
 * switch layout:
 *    1. noContet: Speaker -> Gallery -> Speaker
 *    2. hasContent: Speaker -> Focus -> ContentOnly -> ContentAndAs
 */
void MeetingWindow::onLayoutClicked()
{
    qDebug() << "onLayoutClicked before layout mode: " << meetingLayout->currentLayoutMode();
    if (meetingLayout->currentLayoutMode() == MeetingLayout::Chairman || _totalPeople < 2) {
        // 主会场不允许切换布局
        return;
    }
    int layoutReqNum = 0;
    if (_hasContent) {
        if (meetingLayout->currentLayoutMode() == MeetingLayout::Speaker) {
            meetingLayout->setLayoutMode(MeetingLayout::Focus);
            layoutReqNum = 2;
        }
        else if (meetingLayout->currentLayoutMode() == MeetingLayout::Focus) {
            meetingLayout->setLayoutMode(MeetingLayout::ContentOnly);
            layoutReqNum = 1;
        }
        else if (meetingLayout->currentLayoutMode() == MeetingLayout::ContentOnly) {
            meetingLayout->setLayoutMode(MeetingLayout::ContentAndAs);
            layoutReqNum = 2;
        }
        else /*if (meetingLayout->currentLayoutMode() == MeetingLayout::ContentAndAs)*/ {
            meetingLayout->setLayoutMode(MeetingLayout::Speaker);
            pagelayoutMode = MeetingLayout::Speaker;
            if (_hasContent)
                layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : _totalPeople;
            else
                layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : _totalPeople - 1;
        }
    }
    else {
        if (meetingLayout->currentLayoutMode() == MeetingLayout::Speaker) {
            meetingLayout->setLayoutMode(MeetingLayout::Gallery);
            pagelayoutMode = MeetingLayout::Gallery;
            layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_GALLERT_NUM ? MeetingLayout::LAYOUT_GALLERT_NUM : _totalPeople - 1;
        }
        else {
            pagelayoutMode = MeetingLayout::Speaker;
            meetingLayout->setLayoutMode(MeetingLayout::Speaker);
            layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : _totalPeople - 1;
        }
    }
    adaptor->reqAutoLayout(layoutReqNum, _hasContent, _chirmanUri.toStdString(), meetingLayout->currentLayoutMode());
    meetingLayout->update();
    // rest page mode
    pageIndex = 0;
    qDebug() << "onLayoutClicked after layout mode: " << meetingLayout->currentLayoutMode();
}

void MeetingWindow::parseErrorCode()
{
    QFile josnFile(":/raw/ErrorCode.json");
    if (josnFile.open(QIODevice::ReadOnly)) {
        QByteArray jsonArray = josnFile.readAll();
        QJsonParseError jsonError;
        jsonObj = QJsonDocument::fromJson(jsonArray, &jsonError).object();
        if (jsonError.error != QJsonParseError::NoError)
            qDebug() << "jsonparse json parse got an error";
    }
    josnFile.close();
}

void MeetingWindow::onRosterChanged(const SDKRosterInfo &info)
{
    qDebug() << "MeetingWindow::onRosterChanged";
}

/**
 * @brief MeetingWindow::onStreamChanged 视频流数据变化
 * @param streams 远端视频流
 * 在此处能计算出有多少路视频要渲染, UI上根据自己的业务场景去渲染
 */
void MeetingWindow::onStreamChanged(const std::vector<SDKVideoStreamInfo> &streams)
{
    int streamsSize = streams.size();
    int cellCount = meetingLayout->count() - 1;
    qDebug() << "meeting streamsSize: " << streamsSize << ", cellCount: " << cellCount << ", total count: " << meetingLayout->count();
    if (meetingLayout->currentLayoutMode() == MeetingLayout::Focus ||
            meetingLayout->currentLayoutMode() == MeetingLayout::ContentOnly ||
            meetingLayout->currentLayoutMode() == MeetingLayout::ContentAndAs ||
            (meetingLayout->currentLayoutMode() == MeetingLayout::Gallery && !meetingLayout->isUIShowLocalWhenPageMode())) {
        _localVideoCell->hide();
    } else {
        _localVideoCell->show();
    }

    if (streamsSize > 0) {
        if (streamsSize > cellCount) {
            // add
            for (int i = 0; i < streamsSize - cellCount; i++) {
                VideoCell* videoCell = new VideoCell(this);
                videoCells.append(videoCell);
                meetingLayout->addWidget(videoCell);
            }
        }
        else if (streamsSize == cellCount) {
            // update
        }
        else {
            // delete
            if (!videoCells.isEmpty()) {
                for (int i = 0; i < cellCount - streamsSize; i++) {
                    VideoCell* delCell = videoCells.last();
                    meetingLayout->removeWidget(delCell);
                    delete delCell;
                    videoCells.removeLast();
                }
            }
        }
    }
    else {
        if (!videoCells.isEmpty()) {
            for (int i = 0; i < cellCount - streamsSize; i++) {
                VideoCell* delCell = videoCells.last();
                meetingLayout->removeWidget(delCell);
                delete delCell;
                videoCells.removeLast();
            }
        }
    }
    // update cell state
    for (int i = 0; i < videoCells.size(); i++) {
        videoCells.at(i)->setVidoeInfo(streams.at(i));
    }
}

void MeetingWindow::onVideoFrameReady(const QString &dataSourceId, const SDKVideoFrameWrapper &videoFrame)
{
    if (adaptor->isSdkClosed())
    {
        qDebug() << "sdk has shutdown, cancle it";
        return;
    }
    if (dataSourceId == "LocalPreviewID")
    {
        _localVideoCell->getDisplay()->initBuffer(videoFrame.width * videoFrame.height * 3 / 2);
        _localVideoCell->getDisplay()->requestRender(videoFrame.buffer, videoFrame.bufLen, videoFrame.width, videoFrame.height, videoFrame.sarWidth, videoFrame.sarHeight, videoFrame.rotation, isflip);
    }
    else
    {
        for (VideoCell* cell : videoCells)
        {
            if (cell->getVideoInfo().videoSourceId.str() == dataSourceId.toStdString())
            {
                cell->getDisplay()->initBuffer(videoFrame.width * videoFrame.height * 3 / 2);
                cell->getDisplay()->requestRender(videoFrame.buffer, videoFrame.bufLen, videoFrame.width, videoFrame.height, videoFrame.sarWidth, videoFrame.sarHeight, videoFrame.rotation, videoFrame.flipping);
                break;
            }
        }
    }
}

void MeetingWindow::onVideoStateChanged(SDKVideoStatus status)
{
    QString tips;
    switch (status) {
    case SDKVideoStatus_LowAsLocalBw:
        tips = QString::fromLocal8Bit("本地网络不佳");
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::VideoState, true, tips);
        break;
    case SDKVideoStatus_LowAsRemote:
    case SDKVideoStatus_NetworkError:
    case SDKVideoStatus_LocalWifiIssue:
    case SDKVideoStatus_LowAsLocalHardware:
        tips = QString::fromLocal8Bit("网络状况不佳");
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::VideoState, true, tips);
        break;
    case SDKVideoStatus_Normal:
    default:
        notifyWidget->onNotificationChanged(MeetingNotifyWidget::VideoState, false);
        break;
    }
}

void MeetingWindow::onContentStateChagend(SDKContentState state)
{
    switch (state) {
    case SDKContentState_Idle: {
        if (adaptor->isContentSharing())
        {
            adaptor->stopContentSharing();
        }
        btShare->resetState(QString::fromLocal8Bit("开始共享"), iconShareNormal, iconShareHover, !isContentShareEnable);
        meetingLayout->setLayoutMode(MeetingLayout::Speaker);
        int layoutReqNum;
        if (_hasContent)
            layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : _totalPeople;
        else
            layoutReqNum = _totalPeople > MeetingLayout::LAYOUT_SPEAKER_NUM ? MeetingLayout::LAYOUT_SPEAKER_NUM : _totalPeople - 1;
        adaptor->reqAutoLayout(layoutReqNum, _hasContent, _chirmanUri.toStdString(), MeetingLayout::Speaker);
        meetingLayout->update();
        // content stop swith to normal
        break;
    }
    case SDKContentState_Sending: {
        btShare->resetState(QString::fromLocal8Bit("结束共享"), iconShareNormal, iconShareHover, !isContentShareEnable);
        int layoutReqNum = _totalPeople > 3 ? 3 : _totalPeople;
        meetingLayout->setLayoutMode(MeetingLayout::ContentShare);
        adaptor->reqAutoLayout(layoutReqNum, false, _chirmanUri.toStdString(), MeetingLayout::ContentShare);
        meetingLayout->update();
        // show share widget
        break;
    }
    case SDKContentState_Receiving:
    {
        qDebug("aaaaaaaa");
        break;
    }
    case SDKContentState_NoBandwidth:
        break;
    case SDKContentState_Error:
        break;
    }
}

void MeetingWindow::onNetworkIndicatorLevel(int level)
{
    switch (level) {
    case 1:
        ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_01.png"));
        break;
    case 2:
        ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_02.png"));
        break;
    case 3:
        ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_03.png"));
        break;
    case 4:
        ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_04.png"));
        break;
    default:
        ui->bt_net_level->setIcon(QIcon(":/res/meeting/network_st_04.png"));
    }
}

void MeetingWindow::onKickOut(const QString& code)
{
    qDebug() << "onKickOut : " << code;
    NemoSdkAdaptor::getInstance()->logout();
    adaptor->stopCamera();
    emit sigMeetingFinished("hangup");
}

QWidget *MeetingWindow::getToolbar()
{
    return ui->titlebar;
}

void MeetingWindow::onShow()
{
    this->activateWindow();
    this->showNormal();
}

void MeetingWindow::mouseMoveEvent(QMouseEvent *event)
{
    if (_isPressed) {
        QPoint movePos = event->globalPos() - _position;
        move(movePos);
        event->accept();
    }
}

void MeetingWindow::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        _position = event->pos();
        if (getToolbar() == nullptr) {
            _isPressed = false;
            return;
        }
        if (_position.y() < getToolbar()->height()) {
            _isPressed = true;
        }
    }
}

void MeetingWindow::mouseReleaseEvent(QMouseEvent *)
{
    _isPressed = false;
}

void MeetingWindow::onCloseNetWorkDataShow()
{
    NetWorkDataShowed = false;
}

//事件过滤器
bool MeetingWindow::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == ui->bt_info_show) {
        if (event->type() == QEvent::HoverMove)
        {
            if (_infoshow == nullptr)
            {
                QPoint GlobalPoint(ui->bt_info_show->mapToGlobal(QPoint(0, 0)));//获取控件在窗体中的坐标
                int x = GlobalPoint.x();
                int y = GlobalPoint.y();
                _infoshow = new info_show;
                _infoshow->move(GlobalPoint);
                _infoshow->setGeometry(x - 200, y, 400, 150);
                _infoshow->setStartPos(200);
                _infoshow->setTriangleInfo(20, 12);
                _infoshow->setFixedSize(QSize(400, 150));
                _infoshow->_displayName = this->_userName;
                _infoshow->_meetingNum = this->_meetingNum;
                _infoshow->_shareUrl = this->_shareUrl;
                _infoshow->_password = this->_passWord;
                _infoshow->x = x;
                _infoshow->y = y;

                //_infoshow->show();
            }
            return true;
        }
        else if (event->type() == QEvent::HoverLeave) {
            infoShowNum = 2;
        }
        else
        {

        }
    }
    if (obj == ui->tv_meeting_time)
    {
        if (event->type() == QEvent::HoverMove)
        {
            flagdetailedNetwork = true;
        }
        else if (event->type() == QEvent::HoverLeave) {
            flagdetailedNetwork = false;
            detailedNetworkNum = 0;
        }
        else
        {

        }
    }
    return QWidget::eventFilter(obj, event);
}

void MeetingWindow::onConfHandupCancelled()
{
    Muteflag = 1;
    btMic->setChecked(QString::fromLocal8Bit("举手发言"), iconhandup, iconhanduphover, true);
}

//void MeetingWindow::onRecordResponse(const SDKRecordResponse &response, const QString &reason)
//{
//    switch (response) {
//    case SDK_record_idle:
//        break;
//    case SDK_record_acting:
//        break;
//    case SDK_record_pause_by_others:
//        adaptor->setRecordEnable(false);
//        btRecord->setEnabled(false);
//        btRecord->resetState(QString::fromLocal8Bit("结束录制"), iconRecordStopDisable, iconRecordStopDisable, true);
//        break;
//    case SDK_record_idle_by_others:
//        adaptor->setRecordEnable(true);
//        btRecord->setEnabled(true);
//        btRecord->resetState(QString::fromLocal8Bit("开始录制"), iconRecordNormal, iconRecordHover, false);
//        break;
//    case SDK_record_acting_by_others:
//        adaptor->setRecordEnable(false);
//        btRecord->setEnabled(false);
//        btRecord->resetState(QString::fromLocal8Bit("结束录制"), iconRecordStopDisable, iconRecordStopDisable, true);
//        break;
//    case SDK_record_disable:
//        adaptor->setRecordEnable(false);
//        btRecord->setEnabled(false);
//        btRecord->resetState(QString::fromLocal8Bit("结束录制"), iconRecordStopDisable, iconRecordStopDisable, true);
//        break;
//    }

//}
