﻿#ifndef MEETINGWINDOW_H
#define MEETINGWINDOW_H

#include <QTimer>
#include <QTime>
#include <QJsonObject>

#include "NemoSdkAdaptor.h"
#include "BaseWidget.h"
#include "MeetingStateWidget.h"
#include "MeetingNotifyWidget.h"
#include "FrameLayout.h"
#include "MeetingLayout.h"
#include "VideoCell.h"
#include "MenuButton.h"
#include "NetWorkDataShow.h"
#include "devicemanager.h"
#include "info_show.h"
#include "sharedialog.h"
#include "XYSettings.h"
#include "shareregionwidget.h"
#include "mettinghostwindow.h"

namespace Ui {
class MeetingWindow;
}

class MeetingWindow : public BaseWidget
{
    Q_OBJECT

public:
    explicit MeetingWindow(bool isflipflag=false, QWidget *parent = nullptr);
    ~MeetingWindow();

    const QString iconSettingNormal = ":/res/meeting/more_normal.png";
    const QString iconSettingHover = ":/res/meeting/more_hover.png";
    const QString iconShareNormal = ":/res/meeting/share_normal.png";
    const QString iconShareHover = ":/res/meeting/share_hover.png";
    const QString iconShareDisable = ":/res/meeting/share_disable.png";
    const QString iconFullScreenNormal = ":/res/meeting/full_screen_normal.png";
    const QString iconFullScreenHover = ":/res/meeting/full_screen_hover.png";
    const QString iconHostNormal = ":/res/meeting/meeting_normal.png";
    const QString iconHostHover = ":/res/meeting/meeting_hover.png";
    const QString iconLayoutNormal = ":/res/meeting/switch_layout_normal.png";
    const QString iconLayoutHover = ":/res/meeting/switch_layout_hover.png";
    const QString iconLayoutDisable = ":/res/meeting/switch_layout_disable.png";
    const QString iconRecordNormal = ":/res/meeting/start_record_normal.png";
    const QString iconRecordHover = ":/res/meeting/start_record_hover.png";
    const QString iconRecordStartDisable = ":/res/meeting/start_record_disable.png";
    const QString iconRecordStopDisable = ":/res/meeting/stop_record_disable.png";
    const QString iconRecordStop = ":/res/meeting/stop_record_click.png";
    const QString iconCameraNormal = ":/res/meeting/camera_normal.png";
    const QString iconCameraHover = ":/res/meeting/camera_hover.png";
    const QString iconMicNormal = ":/res/meeting/mic_normal.png";
    const QString iconMicHover = ":/res/meeting/mic_hover.png";
    const QString iconMicLev1 = ":/res/meeting/mic_normal_lev1.png";
    const QString iconMicLev2 = ":/res/meeting/mic_normal_lev2.png";
    const QString iconMicLev3 = ":/res/meeting/mic_normal_lev3.png";
    const QString iconHangupNormal = ":/res/meeting/drop_call_normal.png";
    const QString iconHangupHover = ":/res/meeting/drop_call_normal.png";
    const QString iconCameraClicked = ":/res/meeting/open_camera_click.png";
    const QString iconMicClicked = ":/res/meeting/open_mic_click.png";
    const QString iconhandup = ":/res/meeting/hands_up_normal.png";
    const QString iconhanduphover = ":/res/meeting/hands_up_hover.png";
    const QString iconhanddown = ":/res/meeting/hands_down_normal.png";
    const QString iconhanddownhover = ":/res/meeting/hands_down_hover.png";
    const QString iconendspeak = ":/res/meeting/end_speaker_normal.png";
    const QString iconendspeakhover = ":/res/meeting/end_speaker_hover.png";

public:
    void initToolbar();
    void initBottombar();
    void initClickEvent();
    void onLayoutClicked();
    void parseErrorCode();
    void makeCall(const QString &meetingNumber, const QString &displayName, const QString &password,
                  bool videoMute, bool audioMute, const QString& meetingId);
    void acceptCall(bool videoMute, bool audioMute);
    void onMeetingInvited(const QString& displayName, const SDKCallInfo &callInfo, bool videoMute, bool audioMute);
    bool isflip ;
    bool isLastPage();
    void setUIShowLocalWhenPageMode(bool uiShowLocalWhenPageMode);

protected:
    QWidget* getToolbar();
    void leaveEvent(QEvent *);
    void enterEvent(QEvent *);
    void mouseMoveEvent (QMouseEvent *);
    void mousePressEvent (QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);

signals:
    void sigMeetingFinished(QString reason);

public slots:
    void onShow();

private slots:
    void onConfStateChanged(SDKCallStateInfo callStateInfo);
    void onConfMgmtChanged(const SDKConfMgmtInfo &confMgmtInfo);
    void onConfMemberChanged(int num, bool hasContent, QString chirmanUri);
    void onRosterChanged(const SDKRosterInfo &info);
    void onStreamChanged(const std::vector<SDKVideoStreamInfo> &streams);
    void onVideoFrameReady(const QString &dataSourceId, const SDKVideoFrameWrapper &videoFrame);
    void onVideoStateChanged(SDKVideoStatus status);
    void onContentStateChagend(SDKContentState state);
    void onNetworkIndicatorLevel(int level);
    void onMicEnergyReported(int energy);
    void onKickOut(const QString& code);
    void onTimerEvent();
    void onBottombarTimerEvent();
    void singleShotSlot();
    void onCloseNetWorkDataShow();
    void onConfHandupCancelled();
    //    void onRecordResponse(const SDKRecordResponse&response, const QString& reason);

private:
    Ui::MeetingWindow *ui;
    QPoint _position;
    NemoSdkAdaptor* adaptor = nullptr;
    QList<VideoCell*> videoCells;
    QWidget* videoWidget;
    MeetingStateWidget* meetingStateWidget;
    FrameLayout* frameLayout;
    MeetingNotifyWidget* notifyWidget;
    MeetingLayout* meetingLayout;
    VideoCell* _localVideoCell;
    SDKVideoStreamInfo _localInfo;
    QTimer* timer;
    QTimer* _bottomBarTimer;
    QTime qmeetingTime;
    QString _chirmanUri;
    QString _confMgmtUrl;
    QString _meetingNum;
    QString _displayName;
    QString _userName;
    QString _peopleName;
    QString _shareUrl;
    QString _passWord;
    QString _numberType;
    MenuButton* btSetting;
    MenuButton* btShare;
    MenuButton* btFullScreen;
    MenuButton* btHost;
    MenuButton* btLayou;
    MenuButton* btRecord;
    MenuButton* btCamera;
    MenuButton* btMic;
    MenuButton* btHangup;

    // test
    MenuButton* btFaceLocalFDDetect = nullptr;
    MenuButton* btFaceLocalFDBadge = nullptr;
    MenuButton* btFaceEletronicBadge = nullptr;
    MenuButton* btFaceGetInfo = nullptr;

    SDKCallInfo _inviteCallInfo;
    NetWorkDataShow*OpenNetWorkDataShow = nullptr;

    bool _isConference=false;
    bool _isPressed;
    bool _hasContent;
    bool isContentShareEnable = false;
    bool NetWorkDataShowed=false;
    bool eventFilter(QObject *obj, QEvent *event);
    bool flagdetailedNetwork = false;
    bool _isMakecall;
    bool _isFlip;
    int detailedNetworkNum = 0;
    int infoShowNum=0;
    int volume=0;
    int Muteflag = 0;
    int _totalPeople;
    bool _bugevade=true;
    // json error code
    QJsonObject jsonObj;
    QString _number;
    QString _confMute;

    info_show*_infoshow=nullptr;

    MeetingLayout::LayoutMode pagelayoutMode = MeetingLayout::Speaker;

    ShareRegionWidget* _shareRegWin = nullptr;
    ShareDialog* shareDlg = nullptr;

    bool _isConfHost = false;
    MettingHostWindow* _confHostWindow = nullptr;

    int pageIndex = 0;
    bool _uiShowLocalWhenPageMode = true;

};

#endif // MEETINGWINDOW_H
