﻿#include "NetWorkDataShow.h"
#include "ui_NetWorkDataShow.h"
#include <QScrollBar>
#pragma execution_character_set("utf-8")

NetWorkDataShow::NetWorkDataShow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::NetWorkDataShow)
{
    ui->setupUi(this);

    QFile file(":/stylesheet/style_NetWorkDataShow.qss");
	file.open(QFile::ReadOnly);
	this->setStyleSheet(tr(file.readAll()));
	file.close();

	// 最小化
	connect(ui->bt_min, &QToolButton::clicked, this, [=]() {
		this->showMinimized();
	
	});

	// 关闭
	connect(ui->bt_close, &QToolButton::clicked, this, [=]() {
		this->close();
		emit closeNetWorkDataShow();
	});

	// 最大化
	connect(ui->bt_max, &QToolButton::clicked, this, [=]() {
		if (this->windowState() != Qt::WindowMaximized) {
			this->showMaximized();
		}
		else {
			this->showNormal();
		}
	});
	

	NetDetect = new QTableWidget(0, 6);
    NetDetect->horizontalHeader()->setVisible(false);
    NetDetect->verticalHeader()->setVisible(false);
	NetDetect->setSelectionMode(QAbstractItemView::NoSelection);
	NetDetect->setEditTriggers(QAbstractItemView::NoEditTriggers);
	NetDetect->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
	NetDetect->verticalHeader()->setDefaultSectionSize(40);
	NetDetect->setFocusPolicy(Qt::NoFocus);
    ui->verticalLayout->insertWidget(0,NetDetect);
	

	QTimer *timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(onTimeout()));
	timer->start(1000);
}

NetWorkDataShow::~NetWorkDataShow()
{

    delete ui;
	if (NetDetect != nullptr)
		delete NetDetect;
}
void NetWorkDataShow::onTimeout()
{
	RowCount = 0;//init

    QScrollBar*scrollBar=NetDetect->verticalScrollBar();
    pos=scrollBar->value();


	if(NetDetect!=nullptr)
	delete NetDetect;

    RowCount++;
    NetDetect = new QTableWidget(0, 6);
	NetDetect->horizontalHeader()->setVisible(false);
	NetDetect->verticalHeader()->setVisible(false);
	NetDetect->setSelectionMode(QAbstractItemView::NoSelection);
	NetDetect->setEditTriggers(QAbstractItemView::NoEditTriggers);
	NetDetect->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
	NetDetect->verticalHeader()->setDefaultSectionSize(40);
    NetDetect->setFocusPolicy(Qt::NoFocus);
    ui->verticalLayout->insertWidget(0, NetDetect);

    NetDetect->setRowCount(1);
    NetDetect->setSpan(0, 0, 1, 6);
    QTableWidgetItem*newItem = new QTableWidgetItem("网络探测");
    newItem->setFont(QFont("YouYuan", 13));
    newItem->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    newItem->setTextColor(QColor(168, 168, 168));
    NetDetect->setItem(0, 0, newItem);

	const SDKNetworkStatistics&network = NemoSdkAdaptor::getInstance()->getStatistics().network;
    const QString encrypt = NemoSdkAdaptor::getInstance()->getStatistics().encrypt.str();

	/*network*/
	int Rtt = network.rtt;				/* 往返时间 ms */
	int RxDetectBw = network.rxDetectBw;	/* 下行带宽 kbps */
	int RxJitter = network.rxJitter;		/* 下行抖动 ms */
	int RxLost = network.rxLost;			/* 下行丢包率 */
	int TxDetectBw = network.txDetectBw;	/* 上行带宽 kbps */
	int TxJitter = network.txJitter;		/* 上行抖动 ms */
	int TxLost = network.txLost;			/* 上行丢包率 */

    int sendUsedBW=0;
    const SDKAVSessionStatistics&people=NemoSdkAdaptor::getInstance()->getStatistics().people;
    if(people.videoTxInfo.length()!=0)
    {
        for(int i=0;i<people.videoTxInfo.length();i++){
            sendUsedBW+=people.videoTxInfo[i].actBw;
        }
    }
    if(people.audioTxInfo.length()!=0)
    {
        for(int i=0;i<people.audioTxInfo.length();i++){
            sendUsedBW+=people.audioTxInfo[i].actBw;
        }
    }
    const SDKAVSessionStatistics&content=NemoSdkAdaptor::getInstance()->getStatistics().content;
    if(content.videoTxInfo.length()!=0)
    {
        for(int i=0;i<content.videoTxInfo.length();i++){
            sendUsedBW+=content.videoTxInfo[i].actBw;
        }
    }
    if(content.audioTxInfo.length()!=0)
    {
        for(int i=0;i<content.audioTxInfo.length();i++){
            sendUsedBW+=content.audioTxInfo[i].actBw;
        }
    }

    int recvUsedBW=0;
    if(people.videoRxInfo.length()!=0)
    {
        for(int i=0;i<people.videoRxInfo.length();i++){
            recvUsedBW+=people.videoRxInfo[i].actBw;
        }
    }
    if(people.audioRxInfo.length()!=0)
    {
        for(int i=0;i<people.audioRxInfo.length();i++){
            recvUsedBW+=people.audioRxInfo[i].actBw;
        }
    }
    if(content.videoRxInfo.length()!=0)
    {
        for(int i=0;i<content.videoRxInfo.length();i++){
            recvUsedBW+=content.videoRxInfo[i].actBw;
        }
    }
    if(content.audioRxInfo.length()!=0)
    {
        for(int i=0;i<content.audioRxInfo.length();i++){
            recvUsedBW+=content.audioRxInfo[i].actBw;
        }
    }

	if (array != nullptr)
	delete []array;
    array = new int[5];
	array[0] = RxDetectBw;
    array[1] = sendUsedBW;
    array[2] = RxLost;
    array[3] = Rtt;
    array[4] = RxJitter;
    NetDetect->setRowCount(4);
    RowCount = 4;

    for (int i = 0; i < 6; i++)
	{
		switch (i)
		{
		case 0:  str = "通道名称";     break;
        case 1:  str = "网络带宽(kbps)";    break;
        case 2:  str = "使用带宽(kbps)";    break;
		case 3:  str = "丢包率(%)";  break;
		case 4:  str = "往返延时(ms)"; break;
		case 5:  str = "抖动(ms)"; break;
		default: break;
		}
		QTableWidgetItem*newItem = new QTableWidgetItem(str);
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
        NetDetect->setItem(1, i, newItem);
	}
	int j = 0;
	for (int i = 0; i < 6; i++)
	{
		if (i == 0)
		{
			str = "发送 "; 
		}
		else
		str = QString::number(array[j-1]);
		QTableWidgetItem*newItem = new QTableWidgetItem(str);
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
        NetDetect->setItem(2, i, newItem);
		j++;
	
	}
	if (array != nullptr)
		delete[]array;
    array = new int[5];
	array[0] = TxDetectBw;
    array[1] = recvUsedBW;
    array[2] = TxLost;
    array[3] = Rtt;
    array[4] = TxJitter;
	j = 0;
	for (int i = 0; i < 6; i++)
	{
		if (i == 0)
			str = "接收 ";
		else
			str = QString::number(array[j - 1]);
		QTableWidgetItem*newItem = new QTableWidgetItem(str);
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
        NetDetect->setItem(3, i, newItem);

		j++;
		
	}

	///********************content*************************/
	int audioRxInfolength = content.audioRxInfo.length();/*上行音频统计数据数量*/
	int audioTxInfolength = content.audioTxInfo.length();/*下行音频统计数据数量*/
	int videoRxInfolength = content.videoRxInfo.length();/*上行视频统计数据数量*/
	int videoTxInfolength = content.videoTxInfo.length();/*下行视频统计数据数量*/

	if (audioRxInfolength != 0 || audioTxInfolength != 0 || videoRxInfolength != 0 || videoTxInfolength != 0)//如果有数据则显示
	{
		RowCount++;
		NetDetect->setRowCount(RowCount);
		NetDetect->setSpan(RowCount - 1, 0, 1, 6);

		QTableWidgetItem*newItem = new QTableWidgetItem("内容分享");
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
		NetDetect->setItem(RowCount - 1, 0, newItem);
	

		if (audioRxInfolength != 0)
		{
			for (int i = 0; i < audioRxInfolength; i++)
			{
				XYString stringCodecType = content.audioRxInfo[i].codecType;		/* 编码类型 */
				XYString stringDisplayName = content.audioRxInfo[i].displayName;	/* 显示名称 */
				int  ActBw = content.audioRxInfo[i].actBw;					    /* 实际带宽 */
				RowCount++;
			
				NetDetect->setRowCount(RowCount);
				NetDetect->setSpan(RowCount - 1, 0, 1, 2);
			
				for (int i = 0; i < 6; i++)
				{
					switch (i)
					{
					case 0: str = "音频接收 "; break;
					case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
					case 3: str = "----"; break;
					case 4: str = "----"; break;
					case 5: str = QString::number(ActBw); break;
					}
					QTableWidgetItem*newItem = new QTableWidgetItem(str);
					newItem->setFont(QFont("YouYuan", 13));
					newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
					newItem->setTextColor(QColor(168, 168, 168));
					NetDetect->setItem(RowCount-1, i, newItem);
					if (i == 0)
						i++;
				}
			}
		}
        
		if (audioTxInfolength != 0)
		{
			for (int i = 0; i < audioTxInfolength; i++)
			{
				XYString stringCodecType = content.audioTxInfo[i].codecType;		/* 编码类型 */
				XYString stringDisplayName = content.audioTxInfo[i].displayName;	/* 显示名称 */
				int  ActBw = content.audioTxInfo[i].actBw;						/* 实际带宽 */
				RowCount++;
		
				NetDetect->setRowCount(RowCount);
				NetDetect->setSpan(RowCount - 1, 0, 1, 2);
			
				for (int i = 0; i < 6; i++)
				{
					switch (i)
					{
					case 0: str = "音频发送 "; break;
					case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
					case 3: str = "----"; break;
					case 4: str = "----"; break;
					case 5: str = QString::number(ActBw); break;
					}
					QTableWidgetItem*newItem= new QTableWidgetItem(str);
					newItem->setFont(QFont("YouYuan", 13));
					newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
					newItem->setTextColor(QColor(168, 168, 168));
					NetDetect->setItem(RowCount-1, i, newItem);
					if (i == 0)
						i++;
				}
			}
		}
		if (videoRxInfolength != 0)
		{
			for (int i = 0; i < videoRxInfolength; i++)
			{
				XYString stringCodecType = content.videoRxInfo[i].codecType;		/* 编码类型 */
				XYString stringDisplayName = content.videoRxInfo[i].displayName;	/* 显示名称 */
				XYString stringResolution = content.videoRxInfo[i].resolution;     /* 分辨率 */
				int  ActBw = content.videoRxInfo[i].actBw;						/* 实际带宽 */
				int  frameRate = content.videoRxInfo[i].frameRate;				/* 帧率 */
				RowCount++;
			
				NetDetect->setRowCount(RowCount);
				NetDetect->setSpan(RowCount - 1, 0, 1, 2);
			
				for (int i = 0; i < 6; i++)
				{
					switch (i)
					{
					case 0: str = "视频接收 "; break;
					case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
					case 3:  ch = stringResolution.str(); str = QString(QLatin1String(ch)); break;
					case 4: str = QString::number(frameRate); break;
					case 5: str = QString::number(ActBw); break;
					}
					QTableWidgetItem*newItem = new QTableWidgetItem(str);
					newItem->setFont(QFont("YouYuan", 13));
					newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
					newItem->setTextColor(QColor(168, 168, 168));
					NetDetect->setItem(RowCount-1, i, newItem);
					if (i == 0)
						i++;
				}
			}
		}
		if (videoTxInfolength != 0)
		{
			for (int i = 0; i < videoTxInfolength; i++)
			{
				XYString stringCodecType = content.videoTxInfo[i].codecType;		/* 编码类型 */
				XYString stringDisplayName = content.videoTxInfo[i].displayName;	/* 显示名称 */
				XYString stringResolution = content.videoTxInfo[i].resolution;     /* 分辨率 */
				int  ActBw = content.videoTxInfo[i].actBw;						/* 实际带宽 */
				int  frameRate = content.videoTxInfo[i].frameRate;				/* 帧率 */
				RowCount++;
			
				NetDetect->setRowCount(RowCount);
				NetDetect->setSpan(RowCount - 1, 0, 1, 2);
				
				for (int i = 0; i < 6; i++)
				{
					switch (i)
					{
					case 0: str = "视频发送 "; break;
					case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
					case 3:  ch = stringResolution.str(); str = QString(QLatin1String(ch)); break;
					case 4: str = QString::number(frameRate); break;
					case 5: str = QString::number(ActBw); break;
					}
					QTableWidgetItem*newItem = new QTableWidgetItem(str);
					newItem->setFont(QFont("YouYuan", 13));
					newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
					newItem->setTextColor(QColor(168, 168, 168));
					NetDetect->setItem(RowCount-1, i, newItem);
					if (i == 0)
						i++;
				}
			}
		}
	}
	RowCount++;
	NetDetect->setRowCount(RowCount);
	NetDetect->setSpan(RowCount - 1, 0, 1, 6);

    newItem = new QTableWidgetItem("媒体加密");
	newItem->setFont(QFont("YouYuan", 13));
	newItem->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
	newItem->setTextColor(QColor(168, 168, 168));
	NetDetect->setItem(RowCount - 1, 0, newItem);

	RowCount++;
	NetDetect->setRowCount(RowCount);
	NetDetect->setSpan(RowCount - 1, 0, 1, 3);
	NetDetect->setSpan(RowCount - 1, 3, 1, 3);
	j = 0;
	for (int i = 0; i < 2; i++)
	{
		QString str;
		switch (i)
		{
		case 0:  str = "加密算法";      break;
        case 1:  str = encrypt;         break;
		default: break;
		}
		QTableWidgetItem*newItem = new QTableWidgetItem(str);
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
		NetDetect->setItem(RowCount - 1, j, newItem);
		j = 3;
	
	}

	///*people*/

	RowCount++;
	NetDetect->setRowCount(RowCount);
	NetDetect->setSpan(RowCount - 1, 0, 1, 6);
	newItem = new QTableWidgetItem("与会者");
	newItem->setFont(QFont("YouYuan", 13));
	newItem->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
	newItem->setTextColor(QColor(168, 168, 168));
	NetDetect->setItem(RowCount - 1, 0, newItem);

	RowCount++;
	NetDetect->setRowCount(RowCount);
	
	for (int i = 0; i < 6; i++)
	{
		switch (i)
		{
        case 0:  str = "名称";      break;
		case 1:  str = "通道名称";      break;
		case 2:  str = "编解码器";    break;
		case 3:  str = "分辨率 ";  break;
		case 4:  str = "帧率(fps)"; break;
		case 5:  str = "码率(kbps)"; break;
		default: break;
		}
		QTableWidgetItem*newItem = new QTableWidgetItem(str);
		newItem->setFont(QFont("YouYuan", 13));
		newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
		newItem->setTextColor(QColor(168, 168, 168));
		NetDetect->setItem(RowCount-1, i, newItem);
	
	}

	int PeopleAudioRxInfolength = people.audioRxInfo.length();/*上行音频统计数据数量*/
	if (PeopleAudioRxInfolength != 0)
	{
		for (int i = 0; i < PeopleAudioRxInfolength; i++)
		{
			XYString stringCodecType = people.audioRxInfo[i].codecType;		/* 编码类型 */
			XYString stringDisplayName= people.audioRxInfo[i].displayName;	/* 显示名称 */
			int  ActBw = people.audioRxInfo[i].actBw;					    /* 实际带宽 */

			RowCount++;
			NetDetect->setRowCount(RowCount);
		
			for (int i = 0; i < 6; i++)
			{
				switch (i)
				{
				case 0:  str = stringDisplayName.str();
					arr = str.toUtf8();
					by = arr.fromBase64(arr);
					str = QString::fromUtf8(by);
					break;
				case 1: str = "音频接收 "; break;
				case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
				case 3: str = "----"; break;
				case 4: str = "----"; break;
				case 5: str = QString::number(ActBw); break;
				}
				QTableWidgetItem*newItem = new QTableWidgetItem(str);
				newItem->setFont(QFont("YouYuan", 13));
				newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
				newItem->setTextColor(QColor(168, 168, 168));
				NetDetect->setItem(RowCount-1, i, newItem);
		
			}
		}
	}
	int PeopleAudioTxInfolength = people.audioTxInfo.length();/*下行音频统计数据数量*/
	if (PeopleAudioTxInfolength != 0)
	{
		for (int i = 0; i < PeopleAudioTxInfolength; i++)
		{
			XYString stringCodecType = people.audioTxInfo[i].codecType;		/* 编码类型 */
			XYString stringDisplayName = people.audioTxInfo[i].displayName;	/* 显示名称 */
			int  ActBw = people.audioTxInfo[i].actBw;						/* 实际带宽 */

			RowCount++;
			NetDetect->setRowCount(RowCount);
			for (int i = 0; i < 6; i++)
			{
				
				switch (i)
				{
				    case 0:  str = stringDisplayName.str();
					break;
				case 1: str = "音频发送 "; break;
				case 2:  ch= stringCodecType.str(); str = QString(QLatin1String(ch));break;
				case 3: str = "----"; break;
				case 4: str = "----"; break;
				case 5: str = QString::number(ActBw); break;
				}
				QTableWidgetItem*newItem = new QTableWidgetItem(str);
				newItem->setFont(QFont("YouYuan", 13));
				newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
				newItem->setTextColor(QColor(168, 168, 168));
				NetDetect->setItem(RowCount - 1, i, newItem);
			}
		}
	}
	int PeopleVideoRxInfolength = people.videoRxInfo.length();/*上行视频统计数据数量*/
	if (PeopleVideoRxInfolength != 0)
	{
		for (int i = 0; i < PeopleVideoRxInfolength; i++)
		{
			XYString stringCodecType = people.videoRxInfo[i].codecType;		/* 编码类型 */
			XYString stringDisplayName = people.videoRxInfo[i].displayName;	/* 显示名称 */
			XYString stringResolution = people.videoRxInfo[i].resolution;     /* 分辨率 */
			int  ActBw = people.videoRxInfo[i].actBw;						/* 实际带宽 */
			int  frameRate = people.videoRxInfo[i].frameRate;				/* 帧率 */

		/*	QByteArray arr = stringDisplayName.str();
			QByteArray::fromBase64(arr.toBase64());*/

			RowCount++;
			NetDetect->setRowCount(RowCount);
			for (int i = 0; i < 6; i++)
			{
				switch (i)
				{
				case 0:  str = stringDisplayName.str();
					     arr = str.toUtf8();
						 by = arr.fromBase64(arr);
						 str = QString::fromUtf8(by);
					     break;
				case 1: str = "视频接收 "; break;
				case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
				case 3:  ch = stringResolution.str(); str = QString(QLatin1String(ch)); break;
				case 4: str = QString::number(frameRate); break;
				case 5: str = QString::number(ActBw); break;
				}
				QTableWidgetItem*newItem = new QTableWidgetItem(str);
				newItem->setFont(QFont("YouYuan", 13));
				newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
				newItem->setTextColor(QColor(168, 168, 168));
				NetDetect->setItem(RowCount - 1, i, newItem);
			}
		}
	}
	int PeopleVideoTxInfolength = people.videoTxInfo.length();/*下行视频统计数据数量*/
	if (PeopleVideoTxInfolength != 0)
	{
		for (int i = 0; i < PeopleVideoTxInfolength; i++)
		{
			XYString stringCodecType = people.videoTxInfo[i].codecType;		/* 编码类型 */
			XYString stringDisplayName = people.videoTxInfo[i].displayName;	/* 显示名称 */
			XYString stringResolution = people.videoTxInfo[i].resolution;     /* 分辨率 */
			int  ActBw = people.videoTxInfo[i].actBw;						/* 实际带宽 */
			int  frameRate = people.videoTxInfo[i].frameRate;				/* 帧率 */

			RowCount++;
			NetDetect->setRowCount(RowCount);
			for (int i = 0; i < 6; i++)
			{
				switch (i)
				{
				case 0:  str = stringDisplayName.str();
					break;
				case 1: str = "视频发送 "; break;
				case 2:  ch = stringCodecType.str(); str = QString(QLatin1String(ch)); break;
				case 3:  ch = stringResolution.str(); str = QString(QLatin1String(ch)); break;
				case 4: str = QString::number(frameRate); break;
				case 5: str = QString::number(ActBw); break;
				}
				QTableWidgetItem*newItem = new QTableWidgetItem(str);
				newItem->setFont(QFont("YouYuan", 13));
				newItem->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
				newItem->setTextColor(QColor(168, 168, 168));
				NetDetect->setItem(RowCount - 1, i, newItem);
			}
		}
	}
    if(pos!=0){
        NetDetect->scrollToBottom();
    }
    else {
       NetDetect->scrollToTop();
    }
}
void NetWorkDataShow::mousePressEvent(QMouseEvent *event)
{

	if (event->button() == Qt::LeftButton)
	{
		
			m_startPoint = event->globalPos();
			m_windowPoint = this->frameGeometry().topLeft();
		

	}
}
void NetWorkDataShow::mouseMoveEvent(QMouseEvent *event)
{
	if (event->buttons() & Qt::LeftButton) 
	{
		QPoint relativePos = event->globalPos() - m_startPoint;
		this->move(m_windowPoint + relativePos);
	}
}
void NetWorkDataShow::mouseReleaseEvent(QMouseEvent *event)
{
	if (event->button() == Qt::LeftButton)
	{
	}
}

