﻿#include "shareregionwidget.h"
#include "ui_shareregionwidget.h"

#include <QPainter>
#include <QPen>
#include <QScreen>
#include <QDebug>
#include <QDesktopWidget>

constexpr int RESIZE_THRESHOLD = 10;

ShareRegionWidget::ShareRegionWidget(QWidget *parent) :
    BaseWidget(parent),
    ui(new Ui::ShareRegionWidget)
{
    ui->setupUi(this);

    QDesktopWidget *desktop = QApplication::desktop();
    this->move(QApplication::desktop()->screen()->rect().center() - this->rect().center());

    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setWindowFlags(Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint | Qt::FramelessWindowHint | Qt::SubWindow);

    setMouseTracking(true);
    setAttribute(Qt::WA_Hover);
    installEventFilter(this);

    QRect re = defauleRegion();
    ui->toolbar->setText(QString("RECT[x:%1,y:%2,w:%3,h:%4]").arg(QString::number(re.x()))
                         .arg(QString::number(re.y()))
                         .arg(QString::number(re.width()))
                         .arg(QString::number(re.height())));
}

ShareRegionWidget::~ShareRegionWidget()
{
    delete ui;
}

void ShareRegionWidget::mouseReleaseEvent(QMouseEvent *event)
{
    QPoint point = event->pos();
    qDebug() << __func__ << point.x() << point.y() << this->width() << this->height() << this->x() << this->y();
    _isPressed = false;
    _isResize = false;

    ui->toolbar->setText(QString("RECT[x:%1,y:%2,w:%3,h:%4]").arg(QString::number(this->x()))
                         .arg(QString::number(this->y()))
                         .arg(QString::number(this->width()))
                         .arg(QString::number(this->height())));
    emit sigShareRegChanged(this->x() + 20, this->y() + ui->toolbar->height() + 20, this->width() - 40, this->height() - 40 - ui->toolbar->height());
}

void ShareRegionWidget::mouseMoveEvent(QMouseEvent *event)
{
    if (_isPressed) {
        QPoint movePos = event->globalPos() - _position;
        move(movePos);
        event->accept();
    }
}

void ShareRegionWidget::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        _position = event->pos();
        if (getToolbar() == nullptr) {
            _isPressed = false;
            return;
        }
        if (_position.y() < getToolbar()->height()) {
            _isPressed = true;
        } else {
            _isResize = true;
        }
    }
}

void ShareRegionWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    QPen pen = painter.pen();

    pen.setColor(QColor("#2394DC"));

    painter.setPen(pen);
    painter.drawLine(0, 0, this->width(), 0);

    pen.setWidthF(20);
    painter.setPen(pen);
    painter.drawLine(this->width(), 0, this->width(), this->height());
    painter.drawLine(this->width(), this->height(), 0, this->height());
    painter.drawLine(0, this->height(), 0, 0);
}

QWidget *ShareRegionWidget::getToolbar()
{
    return ui->toolbar;
}

void ShareRegionWidget::handleHoverMove(QEvent *event)
{
    //const QHoverEvent* hoverEvent = static_cast<QHoverEvent*>(event);
    if (_isResize) {
        handleWindowResize(QCursor::pos());
        event->ignore();
        qDebug() << _resizeEdges;
    } else {
        _resizeEdges = testResizeEdge(QCursor::pos());
    }
}

void ShareRegionWidget::handleWindowResize(const QPoint &pos)
{
    QPoint tL = this->geometry().topLeft();
    QPoint bR = this->geometry().bottomRight();

    const int gX = pos.x();
    const int gY = pos.y();

    const bool cRH = bR.x() - tL.x() > this->minimumWidth();
    const bool cRV = bR.y() - tL.y() > this->minimumHeight();

    if (this->_resizeEdges & Qt::TopEdge && (cRV || gY < tL.y())) {
        tL.ry() = gY;
    }

    if (this->_resizeEdges & Qt::BottomEdge && (cRV || gY > bR.y())) {
        bR.ry() = gY;
    }

    if (this->_resizeEdges & Qt::LeftEdge && (cRH || gX < tL.x())) {
        tL.rx() = gX;
    }

    if (this->_resizeEdges & Qt::RightEdge && (cRH || gX > bR.x())) {
        bR.rx() = gX;
    }

    this->setGeometry(QRect(tL, bR));
}

bool ShareRegionWidget::eventFilter(QObject *watched, QEvent *event)
{
    if (event->type() == QEvent::HoverMove) {
        handleHoverMove(event);
    }
    return QWidget::eventFilter(watched, event);
}

Qt::Edges ShareRegionWidget::testResizeEdge(const QPoint &pos) const
{
    const int x = pos.x() - this->x();
    const int y = pos.y() - this->y();
    const int bottom = this->height() - RESIZE_THRESHOLD;
    const int right = this->width() - RESIZE_THRESHOLD;

    Qt::Edges edges = {};

    if (x < RESIZE_THRESHOLD) {
        edges |= Qt::LeftEdge;
    }

    if (y < RESIZE_THRESHOLD) {
        edges |= Qt::TopEdge;
    }

    if (x > right) {
        edges |= Qt::RightEdge;
    }

    if (y > bottom) {
        edges |= Qt::BottomEdge;
    }

    return edges;
}

QRect ShareRegionWidget::defaultPosition()
{
    int width = 820;
    int height = 460;
    QPoint center = QGuiApplication::primaryScreen()->geometry().center();
    return QRect(center.x() - width * .5f, center.y() - height * .5f, width, height);
}

QRect ShareRegionWidget::defauleRegion()
{
    QRect rect = defaultPosition();
    return QRect(rect.x() + 20, rect.y() + + ui->toolbar->height() + 20, rect.width() - 40, rect.height()- 40 - ui->toolbar->height());
}
